# -*- coding: utf-8 -*-

__author__ = """Thomas Milliman"""
__email__ = 'thomas.milliman@unh.edu'
__version__ = '0.1.1rc2'

import os

from . import config

# from roimask import ROIMask
# from roilist import ROIList
# from roitimeseries import ROITimeSeries
# from gcctimeseries import GCCTimeSeries
# from quantile import quantile
# from vegindex import get_roi_list, get_roi_timeseries, \
#      get_gcc_timeseries, daterange2
# from utils import getsiteimglist

# root directory of data archive
if os.environ.get('PHENOCAM_ARCHIVE_DIR'):
    config.archive_dir = os.environ.get('PHENOCAM_ARCHIVE_DIR')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_roilist
----------------------------------

Tests for `vegindex.roilist` module.
"""

import os

import pytest

from vegindex import config
from vegindex import roilist

# MODULE_DIR = os.path.dirname(vegindex.__file__)
SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__),
                               'sample_data')


def test_roilist_creation():
    """
    basic ROIlist creation
    """

    mylist = roilist.ROIList(site='testsite',
                             roitype='DB',
                             descrip='test ROI',
                             sequence_number=1,
                             owner='phenocam')

    assert mylist.roitype == 'DB'
    assert mylist.site == 'testsite'
    assert mylist.sequence_number == 1
    assert mylist.owner == 'phenocam'
    assert mylist.namestring() == 'testsite_DB_0001'
    assert mylist.__unicode__() == u'testsite_DB_0001'
    assert len(mylist.masks) == 0


def test_roilist_vegtype():
    """
    try creating a list with an unsupported veg_type
    """

    with pytest.raises(ValueError):
        mylist = roilist.ROIList(site='testsite',
                                 roitype='DE',
                                 descrip='test ROI',
                                 sequence_number=1,
                                 owner='phenocam')
        print mylist


def test_roilist_readcsv():
    """
    test reading a sample ROIList file
    """

    roilist_site = 'harvard'
    roilist_file = 'harvard_DB_0001_roi.csv'
    roilist_path = os.path.join(SAMPLE_DATA_DIR,
                                roilist_site,
                                'ROI',
                                roilist_file)

    mylist = roilist.ROIList()
    mylist.readCSV(roilist_path)
    assert mylist.site == 'harvard'
    assert mylist.roitype == 'DB'
    assert mylist.sequence_number == 1
    assert len(mylist.masks) == 1



# -*- coding: utf-8 -*-

"""
Python Module for Vegetation Index Generation Routines
"""

import os
from datetime import date
from datetime import timedelta

from . import config
from .gcctimeseries import GCCTimeSeries
from .roilist import ROIList
from .roitimeseries import ROITimeSeries

# ********** Public Functions **************


def daterange2(start_date, end_date, nday):
    """
    make alternate daterange() function which uses fixed days
    of year between start_date and end_date.  In other words
    if nday=3 we always want the dates for doy=1,4,7, ...
    which fall between start and end dates.  Note: we start
    on the interval that includes the first date.
    """
    year_start = start_date.year
    year_end = end_date.year

    # loop over years and always use same doy intervals ...
    for iyear in range(year_start, year_end + 1, 1):

        # if this is the first year reset the start
        # date to the beginning of the first nday period
        # which includes start_date
        if iyear == year_start:
            doy_start = (start_date - date(iyear, 1, 1)).days + 1
            n = (doy_start - 1) / nday
            new_doy_start = n * nday + 1
            dstart = date(iyear, 1, 1) + timedelta(new_doy_start - 1)
        else:
            # start at the first day of year
            dstart = date(iyear, 1, 1)

        # if this is the last year, find the beginning
        # of the last nday period before the last date
        if iyear == year_end:
            doy_last = (end_date -
                        date(iyear, 1, 1)).days + 1
            n = (doy_last - 1) / nday
            new_doy_last = n * nday + 1
            dlast = date(iyear, 1, 1) + timedelta(new_doy_last - 1)

        else:
            doy_last = (date(iyear, 12, 31) -
                        date(iyear, 1, 1)).days + 1
            dlast = date(iyear, 12, 31)

        # for this year yield the first day of the next nday period
        for n in range(0, doy_last + 1, nday):
            dtval = dstart + timedelta(n)
            if ((dtval + timedelta(nday)) > start_date) and (dtval <= dlast):
                yield dtval


def get_roi_list(site, roilist_id):
    """
    function to read in CSV ROI file and return a ROIList object.
    """

    # take ROIList_id and parse into site, roitype, sequence_number
    (roitype, seqno_str) = roilist_id.split('_')
    sequence_number = int(seqno_str)

    # set cannonical dir for ROI Lists
    roidir = os.path.join(config.archive_dir, site, 'ROI')

    # set cannonical filename
    roifile = site + '_' + roilist_id + '_roi.csv'
    roipath = os.path.join(roidir, roifile)

    # create ROIList object
    roilist = ROIList(site=site, roitype=roitype,
                      sequence_number=sequence_number)

    # read in from CSV file
    roilist.readCSV(roipath)

    return roilist


def get_roi_timeseries(site, roilist_id):
    """
    function to read in CSV ROI file and return a ROITimeSeries object
    """

    # take ROIList_id and parse into site, roitype, sequence_number
    (roitype, seqno_str) = roilist_id.split('_')

    # set cannonical dir for ROI Lists
    roidir = os.path.join(config.archive_dir, site, 'ROI')

    # set cannonical filename
    roitsfile = site + '_' + roilist_id + '_timeseries.csv'
    roitspath = os.path.join(roidir, roitsfile)
    # print roitspath

    # create empty ROITimeSeries object
    roits = ROITimeSeries(site=site, ROIListID=roilist_id)

    # read in from CSV file
    roits.readCSV(roitspath)

    return roits


def get_gcc_timeseries(site, roilist_id, nday=3):
    """
    Read in CSV version of summary timeseries and return
    GCCTimeSeries object.
    """

    # take ROIList_id and parse into roitype, sequence_number
    (roitype, seqno_str) = roilist_id.split('_')

    # set cannonical dir for ROI Lists
    roidir = os.path.join(config.archive_dir, site, 'ROI')

    # set cannonical filename
    gcc_tsfile = site + '_' + roilist_id + '_{0}day.csv'.format(nday)
    gcc_tspath = os.path.join(roidir, gcc_tsfile)

    # create empty GCCTimeSeries object
    gccts = GCCTimeSeries(site=site, ROIListID=roilist_id)

    # read in from CSV file
    gccts.readCSV(gcc_tspath)

    return gccts



# -*- coding: utf-8 -*-

"""
Command line script to generate a ROI timeseries CSV from images for a
particular site and ROI.  All available images will be include.

"""

import argparse
import os
import sys
from ConfigParser import SafeConfigParser

import numpy as np
from PIL import Image

import vegindex as vi

from . import utils

# set vars

# you can set the archive directory to somewhere else for testing by
# using the env variable, PHENOCAM_ARCHIVE_DIR.
archive_dir = vi.config.archive_dir

debug = False
default_resize = vi.config.RESIZE


# if __name__ == "__main__":
def main():
    """
    generate ROI timeseries from a PhenoCam directory of images
    """

    # set up command line argument processing
    parser = argparse.ArgumentParser()

    # options
    parser.add_argument("-v", "--verbose",
                        help="increase output verbosity",
                        action="store_true",
                        default=False)
    parser.add_argument("-n", "--dry-run",
                        help="Process data but don't save results",
                        action="store_true",
                        default=False)

    # positional arguments
    parser.add_argument("site", help="PhenoCam site name")
    parser.add_argument("roiname", help="ROI name, e.g. DB_0001")

    # get args
    args = parser.parse_args()
    sitename = args.site
    roiname = args.roiname
    verbose = args.verbose
    dryrun = args.dry_run

    if verbose:
        print "site: {0}".format(sitename)
        print "roiname: {0}".format(roiname)
        print "verbose: {0}".format(verbose)
        print "dryrun: {0}".format(dryrun)

    # set output filename
    outname = '%s_%s_timeseries.csv' % (sitename, roiname,)
    outdir = os.path.join(archive_dir, sitename, 'ROI')
    outpath = os.path.join(outdir, outname)
    if verbose:
        print "archive dir: {0}".format(archive_dir)
        print "output file: {0}".format(outname)

    # read in config file for this site if it exists
    config_file = "{0}_{1}.cfg".format(sitename, roiname)
    config_path = os.path.join(archive_dir, sitename, 'ROI',
                               config_file)
    if os.path.exists(config_path):
        cfgparser = SafeConfigParser(
            defaults={'resize': str(default_resize)})
        cfgparser.read(config_path)
        if cfgparser.has_section('roi_timeseries'):
            resizeFlg = cfgparser.getboolean('roi_timeseries', 'resize')
        else:
            resizeFlg = default_resize

    else:
        resizeFlg = default_resize

    # print config values
    if verbose:
        print ''
        print 'ROI timeseries config:'
        print '======================'
        print 'roi_list: ', '{0}_{1}_roi.csv'.format(sitename, roiname)
        if os.path.exists(config_path):
            print "config file: {0}".format(config_file)
        else:
            print "config file: None"
        print 'Resize Flag: ', resizeFlg

    # create new roi_timeseries object for this ROIList
    roits = vi.ROITimeSeries(site=sitename, ROIListID=roiname,
                             resizeFlag=resizeFlg)

    # grab roi list
    roi_list = vi.get_roi_list(sitename, roiname)

    # loop over mask entries in ROI list
    nimage = 0
    nupdate = 0
    for roimask_index, roimask in enumerate(roi_list.masks):

        startDT = roimask['start_dt']
        endDT = roimask['end_dt']
        maskfile = roimask['maskfile']

        mask_path = os.path.join(archive_dir, sitename, 'ROI', maskfile)
        # open roi mask file
        try:
            mask_img = Image.open(mask_path)

        except:
            sys.stderr.write("Unable to open ROI mask file\n")
            sys.exit(1)

        # check that mask_img is in expected form
        mask_mode = mask_img.mode
        if mask_mode != 'L':

            # convert to 8-bit mask
            mask_img = mask_img.convert('L')

        # make a numpy mask
        roimask = np.asarray(mask_img, dtype=np.bool8)

        # get list of images for this timeperiod
        imglist = utils.getsiteimglist(sitename, getIR=False,
                                       startDT=startDT, endDT=endDT)

        nimage += len(imglist)

        for impath in imglist:

            # append row for this image/mask - shouldn't get
            # any duplicates so just append
            roits_row = roits.append_row(impath, roimask,
                                         roimask_index + 1)
            if roits_row:
                nupdate += 1
            else:
                continue

            if verbose:
                csvstr = roits.format_csvrow(roits_row)
                print csvstr

            if debug:
                if nupdate == 10:
                    break

    # output CSV file
    if dryrun:
        nout = 0
    else:
        nout = roits.writeCSV(outpath)

    print 'Images processed: %d' % (nimage,)
    print 'Images added to CSV: %d' % (nupdate,)
    print 'Total: %d' % (nout,)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
Tests for `vegindex.utils` module.
"""

from vegindex import utils


def test_get_siteinfo():
    """
    get getting site info from URL
    """

    sitename = 'harvard'
    siteinfo = utils.getsiteinfo(sitename)

    assert siteinfo['lon'] == -72.1715



"""
File    quantile.py
Desc    computes sample quantiles
Author  Ernesto P. Adorio, PhD.
                UPDEPP (U.P. at Clarkfield)
Version 0.0.1 August 7. 2009
"""

from math import floor
from math import modf


def quantile(x, q, qtype=7, issorted=False):
    """
    Args:
        x - input data
        q - quantile
        qtype - algorithm
        issorted- True if x already sorted.

    Compute quantiles from input array x given q.For median,
    specify q=0.5.

    References:
        http://reference.wolfram.com/mathematica/ref/Quantile.html
        http://wiki.r-project.org/rwiki/doku.php?id=rdoc:stats:quantile

    Author:
        Ernesto P.Adorio Ph.D.
        UP Extension Program in Pampanga, Clark Field.
    """

    if not issorted:
        y = sorted(x)
    else:
        y = x
        if not (1 <= qtype <= 9):
            return None  # error!

    # Parameters for the Hyndman and Fan algorithm
    abcd = [
        # inverse empirical distrib.function., R type 1
        (0,   0, 1, 0),
        # similar to type 1, averaged, R type 2
        (0.5, 0, 1, 0),
        # nearest order statistic,(SAS) R type 3
        (0.5, 0, 0, 0),
        # California linear interpolation, R type 4
        (0,   0, 0, 1),
        # hydrologists method, R type 5
        (0.5, 0, 0, 1),
        # mean-based estimate(Weibull method), (SPSS, Minitab), type 6
        (0,   1, 0, 1),
        # mode-based method,(S, S-Plus), R type 7
        (1,  -1, 0, 1),
        # median-unbiased ,  R type 8
        (1.0 / 3, 1.0 / 3, 0, 1),
        # normal-unbiased, R type 9.
        (3 / 8.0, 0.25, 0, 1)
    ]

    a, b, c, d = abcd[qtype - 1]
    n = len(x)
    g, j = modf(a + (n + b) * q - 1)
    if j < 0:
        return y[0]
    elif j >= n:
        # oct. 8, 2010 y[n]???!! uncaught  off by 1 error!!!
        return y[n - 1]

    j = int(floor(j))
    if g == 0:
        return y[j]
    else:
        return y[j] + (y[j + 1] - y[j]) * (c + d * g)


def Test():
    x = [11.4, 17.3, 21.3, 25.9, 40.1, 50.5, 60.0, 70.0, 75]

    for qtype in range(1, 10):
        print qtype, quantile(x, 0.35, qtype)


if __name__ == "__main__":
    Test()



# -*- coding: utf-8 -*-

import sys

import numpy as np
from PIL import Image


class ROIMask(object):
    """
    Class for ROI Mask File
    """

    def __init__(self, ROIList_id, start_dt, end_dt, maskfile, sample_image):
        """
        Create ROIMask object - right now there is no database entry for
        an individual mask file but we might add one at some point.  That
        way we could have a form for creating a ROIList complete with
        a list of masks.  Right now we just create an dummy entry.

        At some point we also might want to add a comment field so we
        know why the new mask was added ... initial mask, FOV shift,
        avoid new ref. panel, etc
        """

        self.ROIList_id = ROIList_id
        self.start_dt = start_dt
        self.end_dt = end_dt
        self.maskfile = maskfile
        self.sample_image = sample_image

        # get sitename, roi-type, sequence number from ID
        [s1, s2, s3] = self.ROIList_id.split('_')
        self.site = s1
        self.roitype = s2
        self.sequence_number = int(s3)

    def formatCSVRow(self):
        """
        Return a string which can be used as a row in an
        ROIList CSV file.
        """

        # CSV has start_date, start_time, end_date, end_time,
        # maskfile, sample_image
        start_date = self.start_dt.date()
        start_time = self.start_dt.time()
        end_date = self.end_dt.date()
        end_time = self.end_dt.time()
        csvrow = '{0},{1},{2},{3},{4},{5}'.format(start_date, start_time,
                                                  end_date, end_time,
                                                  self.maskfile,
                                                  self.sample_image)
        return csvrow

    def read(self, roiMaskPath):
        """
        Read ROIMask file and return a numpy array.
        """

        # read using PIL
        try:
            mask_img = Image.open(roiMaskPath)
        except:
            sys.stderr.write("Unable to open mask file\n")
            return None

        # convert to numpy boolean mask
        mask = np.asarray(mask_img, dtype=np.bool8)

        return mask



# -*- coding: utf-8 -*-

# flag for dealing with IR images - by default
# we're not looking at IR images.
irFlag = False

# set default archive dir
archive_dir = '/data/archive'

# set up list of predefined ROITypes
ROITypes = ['canopy',
            'deciduous',
            'coniferous',
            'grass',
            'shrub',
            'modis',
            'refpanel',
            'misc',
            'tundra',
            'AG',
            'DB',
            'DN',
            'EB',
            'EN',
            'GR',
            'MX',
            'NV',
            'RF',
            'SH',
            'TN',
            'UN',
            'WL',
            'XX']

# set up image selection criteria for summary (gcc90 1-day
# and gcc90 3-day) files.

# set up selection for minimum Sun Elevation
MIN_SUN_ANGLE = 10.0

# set up selection for minimum and maximum brightness
MIN_BRT = 100
MAX_BRT = 665

# set up time-of-day restrictions
TIME_MIN = '00:00:00'
TIME_MAX = '23:59:59'

# set up minimum number of images for summary period
NIMAGE_MIN = 1

# set up default resize behavior
RESIZE = False

# set up no/missing data values
ND_FLOAT = 'NA'
ND_INT = 'NA'
ND_STRING = "None"

# ND_FLOAT = '-9999.'
# ND_INT = '-9999'
# ND_STRING = "None"



# -*- coding: utf-8 -*-

import config
import csv
import os
import re
import sys
from datetime import datetime

import numpy as np
from PIL import Image

from . import utils

ND_FLOAT = config.ND_FLOAT
ND_INT = config.ND_INT
ND_STRING = config.ND_STRING


def _float_or_none(str):
    """
    Try to convert string to float. Return float or None value if a
    ValueError exception is raised.
    """
    try:
        retval = float(str)
    except ValueError:
        retval = ND_FLOAT

    if retval == -9999.:
        retval = ND_FLOAT

    return retval


def _int_or_none(str):
    """
    Try to convert string to integer. Return int or None value
    if ValueError is raised.
    """
    try:
        retval = int(str)
    except ValueError:
        retval = ND_INT

    if retval == -9999:
        retval = ND_INT

    return retval

######################################################################


def get_dn_means(im, roimask):
    """
    Function to return mean DN values for an image / mask pair.
    NOTE: probably move this to utils.py.
    """

    # split into bands
    try:
        (im_r, im_g, im_b) = im.split()
    except:
        sys.stderr.write("Wrong image type\n")
        return None

    # create numpy arrays with bands
    r_array = np.asarray(im_r, dtype=np.float64)
    g_array = np.asarray(im_g, dtype=np.float64)
    b_array = np.asarray(im_b, dtype=np.float64)
    brt_array = r_array + g_array + b_array

    # try applying mask to red image ... if mask and image don't
    # have same size this will raise an exception.
    try:
        r_ma = np.ma.array(r_array, mask=roimask)
    except:
        errstr = "Error applying mask to image file.\n"
        sys.stderr.write(errstr)
        return None

    # make masked arrays for G,B
    g_ma = np.ma.array(g_array, mask=roimask)
    b_ma = np.ma.array(b_array, mask=roimask)

    # find mean values to store
    brt = np.mean(brt_array)
    g_mean_roi = np.mean(g_ma)
    r_mean_roi = np.mean(r_ma)
    b_mean_roi = np.mean(b_ma)

    return [r_mean_roi, g_mean_roi,  b_mean_roi, brt]


def get_roi_stats(im, roimask):
    """
    Function to return a more extensive collection of stats for DN
    values for an image / mask pair.  NOTE: probably move this to
    utils.py.
    """

    # split into bands
    try:
        (im_r, im_g, im_b) = im.split()
    except:
        sys.stderr.write("Wrong image type\n")
        return None

    # create numpy arrays with bands
    r_array = np.asarray(im_r, dtype=np.int16)
    g_array = np.asarray(im_g, dtype=np.int16)
    b_array = np.asarray(im_b, dtype=np.int16)
    brt_array = r_array + g_array + b_array

    # check that the image isn't nearly all black or all white in
    # which case getting the stats fails.  Eliminate the outer 30
    # pixels in case there is a banner/overlay on the image.
    #
    # NOTE: this is using almost entire image not just ROI

    if brt_array[30:-30, 30:-30].mean() < 30.0:
        warningstr = "WARNING: mostly dark image.\n"
        sys.stderr.write(warningstr)
        r_mean = ND_FLOAT
        r_std = ND_FLOAT
        r_pcts = [ND_FLOAT, ND_FLOAT, ND_FLOAT, ND_FLOAT,
                  ND_FLOAT, ND_FLOAT, ND_FLOAT]
        g_mean = ND_FLOAT
        g_std = ND_FLOAT
        g_pcts = [ND_FLOAT, ND_FLOAT, ND_FLOAT, ND_FLOAT,
                  ND_FLOAT, ND_FLOAT, ND_FLOAT]
        b_mean = ND_FLOAT
        b_std = ND_FLOAT
        b_pcts = [ND_FLOAT, ND_FLOAT, ND_FLOAT, ND_FLOAT,
                  ND_FLOAT, ND_FLOAT, ND_FLOAT]
        RG_cor = ND_FLOAT
        GB_cor = ND_FLOAT
        BR_cor = ND_FLOAT
        return [{'mean': r_mean,
                 'stdev': r_std,
                 'percentiles': r_pcts},
                {'mean': g_mean,
                 'stdev': g_std,
                 'percentiles': g_pcts},
                {'mean': b_mean,
                 'stdev': b_std,
                 'percentiles': b_pcts},
                RG_cor, GB_cor, BR_cor]

    if brt_array[30:-30, 30:-30].mean() > 725.0:
        warningstr = "WARNING: mostly white image.\n"
        sys.stderr.write(warningstr)
        r_mean = ND_FLOAT
        r_std = ND_FLOAT
        r_pcts = [ND_FLOAT, ND_FLOAT, ND_FLOAT, ND_FLOAT,
                  ND_FLOAT, ND_FLOAT, ND_FLOAT]
        g_mean = ND_FLOAT
        g_std = ND_FLOAT
        g_pcts = [ND_FLOAT, ND_FLOAT, ND_FLOAT, ND_FLOAT,
                  ND_FLOAT, ND_FLOAT, ND_FLOAT]
        b_mean = ND_FLOAT
        b_std = ND_FLOAT
        b_pcts = [ND_FLOAT, ND_FLOAT, ND_FLOAT, ND_FLOAT,
                  ND_FLOAT, ND_FLOAT, ND_FLOAT]
        RG_cor = ND_FLOAT
        GB_cor = ND_FLOAT
        BR_cor = ND_FLOAT
        return [{'mean': r_mean,
                 'stdev': r_std,
                 'percentiles': r_pcts},
                {'mean': g_mean,
                 'stdev': g_std,
                 'percentiles': g_pcts},
                {'mean': b_mean,
                 'stdev': b_std,
                 'percentiles': b_pcts},
                RG_cor, GB_cor, BR_cor]

    # try applying mask to red image ... if mask and image don't
    # have same size this will raise an exception.
    try:
        r_ma = np.ma.array(r_array, mask=roimask)
    except Exception as inst:
        print inst
        errstr = "Error applying mask to image file.\n"
        sys.stderr.write(errstr)
        return None

    # make masked arrays for G,B
    g_ma = np.ma.array(g_array, mask=roimask)
    b_ma = np.ma.array(b_array, mask=roimask)

    # find mean and std values
    r_vals = r_ma.compressed()
    r_mean = r_vals.mean()
    r_diff = np.float64(r_vals) - r_mean
    r_std = np.sqrt(np.dot(r_diff, r_diff) / r_vals.size)

    g_vals = g_ma.compressed()
    g_mean = g_vals.mean()
    g_diff = np.float64(g_vals) - g_mean
    g_std = np.sqrt(np.dot(g_diff, g_diff) / g_vals.size)

    b_vals = b_ma.compressed()
    b_mean = b_vals.mean()
    b_diff = np.float64(b_vals) - b_mean
    b_std = np.sqrt(np.dot(b_diff, b_diff) / b_vals.size)

    # calculate percentiles for each array
    r_pcts = np.percentile(r_vals, (5., 10., 25., 50., 75., 90., 95.))
    g_pcts = np.percentile(g_vals, (5., 10., 25., 50., 75., 90., 95.))
    b_pcts = np.percentile(b_vals, (5., 10., 25., 50., 75., 90., 95.))

    # calculate covariance matrices and get cross-diagonal terms
    rg_cov = np.dot(r_diff, g_diff) / r_diff.size
    gb_cov = np.dot(g_diff, b_diff) / g_diff.size
    br_cov = np.dot(b_diff, r_diff) / b_diff.size

    # calculate correlation coefficient
    RG_cor = rg_cov / (r_std * g_std)
    GB_cor = gb_cov / (g_std * b_std)
    BR_cor = br_cov / (b_std * r_std)

    # if the above calculation can return a NaN then
    # I should probably trap that here and return a

    # return list of values
    return [{'mean': r_mean,
             'stdev': r_std,
             'percentiles': r_pcts},
            {'mean': g_mean,
             'stdev': g_std,
             'percentiles': g_pcts},
            {'mean': b_mean,
             'stdev': b_std,
             'percentiles': b_pcts},
            RG_cor, GB_cor, BR_cor]


def _filter_comments(f):
    """
    filter comments from csv file
    """
    for l in f:
        line = l.rstrip()
        if line and not line.startswith("#"):
            yield line


def _get_comments(f):
    """
    return JUST the comment lines from a csv file
    """
    for l in f:
        line = l.rstrip()
        if line and line.startswith("#"):
            yield line


def _get_comment_field(comments, var_string):
    """
    return value of a field from a list of comment lines

    The comment line with a field has the following format:

    # var_string: var_value

    For example:

    # Creation Date: 2012-02-01

    So we need to create a pattern, find a match and then return
    the value.  No checking of whether the value is valid will
    be done by this routine.
    """

    # set pattern to match
    pattern = '# {0}:\ (?P<var_value>.+)$'.format(var_string)

    var_value = ""
    for line in comments:

        result = re.match(pattern, line)

        if result is not None:
            var_value = result.group('var_value')
            break

    return var_value


class ROITimeSeries(object):
    """
    Class for CSV version of ROI Timeseries.  There is
    currently no equivalent dB table for an ROI time series
    object.

    Here's a proposed sample ROITimeSeries CSV file format:

        #
        # All-image time series for arbutuslake
        #
        # Site: arbutuslake
        # Veg Type: DB
        # ROI ID Number: arbutuslake_canopy_0001
        # Lat:
        # Lon:
        # Elev:
        # UTC Offset:
        # Resize Flag: False
        # Version: 1
        # Creation Date: 2013-03-29
        # Creation Time: 12:10:59
        # Update Date: 2013-03-29
        # Update Time: 14:16:43
        #
        date,local_std_time,doy,filename,solar_elev,exposure,mask_index,gcc,rcc,\
        r_mean,r_std,r_5_qtl,r_10_qtl,r_25_qtl,r_50_qtl,r_75_qtl,r_90_qtl,r_95_qtl,\
        g_mean,g_std,g_5_qtl,g_10_qtl,g_25_qtl,g_50_qtl,g_75_qtl,g_90_qtl,g_95_qtl,\
        b_mean,b_std,b_5_qtl,b_10_qtl,b_25_qtl,b_50_qtl,b_75_qtl,b_90_qtl,b_95_qtl,\
        r_g_correl,g_b_correl,b_r_correl
        2014-05-06,11:01:33,126,coweeta_2014_05_06_110133.jpg,62.96,-9999,2,
        92.25984,46.95709,24,39,62,86,116,154,181,\
        97.92024,44.42619,31,46,69,94,120,153,180,\
        31.40147,32.83882,0,0,4,25,45,72,95,\
        0.96995,0.89691,0.91008

    """

    def __init__(self, site='', ROIListID='', resizeFlag=False):
        """
        create ROITimeSeries object
        """

        self.site = site
        self.roilistid = ROIListID
        # self.irFlg = irFlag
        self.resizeFlg = resizeFlag
        self.created_at = datetime.now()
        self.updated_at = datetime.now()
        self.rows = []

        # split ROIListID into roitype, and sequence_number
        roitype, sequence_number = ROIListID.split('_')
        self.roitype = roitype
        self.sequence_number = int(sequence_number)

        # grab site info ... really should be a separate
        # class for site with all the site info ... should
        # figure out how to use Django classes here
        if site:
            try:
                si = utils.getsiteinfo(site)
            except:
                si = {'lat': None,
                      'lon': None,
                      'elev': None,
                      'tzoffset': None}
        else:
            si = {'lat': None,
                  'lon': None,
                  'elev': None,
                  'tzoffset': None}

        self.lat = si['lat']
        self.lon = si['lon']
        self.elev = si['elev']
        self.tzoffset = si['tzoffset']

    def get_image_list(self):
        """
        return list of images in an ROI timeseries
        """
        imglist = [row['filename'] for row in self.rows]

        return imglist

    def create_row(self, impath, roimask, mask_index):
        """
        create a ROITimeSeries row dictionary for a given image and
        ROI mask.
        """

        # extract datetime from filename
        img_file = os.path.basename(impath)
        img_DT = utils.fn2datetime(self.site, img_file, irFlag=False)
        img_date = img_DT.date()
        img_time = img_DT.time()

        # get doy
        # img_doy = img_DT.timetuple().tm_yday

        # find sun elevation (degrees)
        sun_elev = utils.sunelev(self.lat, self.lon, img_DT, self.tzoffset)

        # Try to load image
        try:
            im = Image.open(impath, 'r')
            im.load()
        except:
            errstr1 = "Unable to open file: %s\n" % (impath,)
            errstr2 = "Skipping this file.\n"
            sys.stderr.write(errstr1)
            sys.stderr.write(errstr2)
            return None

        # if resizeFlg is True resize image to match mask
        if self.resizeFlg:
            ysize, xsize = roimask.shape
            if (xsize, ysize) != im.size:
                warnmsg = "Resizing image {0} to match mask.\n"
                warnmsg = warnmsg.format(img_file)
                sys.stdout.write(warnmsg)
                im = im.resize((xsize, ysize), Image.ANTIALIAS)

        # find mean values over ROI
        try:
            # [dn_r, dn_g, dn_b, brt] = get_dn_means(im, roimask)
            roistats_list = get_roi_stats(im, roimask)

        except KeyboardInterrupt:
            sys.exit()
        except Exception as inst:
            print inst
            errstr1 = "Problem getting ROI " + \
                      "stats for file in create_row(): {0}\n".format(impath)
            sys.stderr.write(errstr1)
            return None

        # get_dn_stats returns None if can't get stats (probably
        # should just throw exception in which case it would be caught
        # by above except clause

        if not(roistats_list):
            return None

        # extract stats
        r_stats = roistats_list[0]
        g_stats = roistats_list[1]
        b_stats = roistats_list[2]
        rg_cor = roistats_list[3]
        gb_cor = roistats_list[4]
        br_cor = roistats_list[5]

        r_mean = r_stats['mean']
        r_std = r_stats['stdev']
        # r_pcts = r_stats['percentiles']
        r_5, r_10, r_25, r_50, r_75, r_90, r_95 = r_stats['percentiles']

        g_mean = g_stats['mean']
        g_std = g_stats['stdev']
        # g_pcts = g_stats['percentiles']
        g_5, g_10, g_25, g_50, g_75, g_90, g_95 = g_stats['percentiles']

        b_mean = b_stats['mean']
        b_std = b_stats['stdev']
        # b_pcts = b_stats['percentiles']
        b_5, b_10, b_25, b_50, b_75, b_90, b_95 = b_stats['percentiles']

        if r_mean != ND_FLOAT:
            brt_mean = r_mean + g_mean + b_mean
        else:
            brt_mean = ND_FLOAT

        if brt_mean > 0 and brt_mean != ND_FLOAT:
            gcc = g_mean / brt_mean
            rcc = r_mean / brt_mean
        else:
            gcc = ND_FLOAT
            rcc = ND_FLOAT

        # get exposure - just set to missing value for now
        exposure = ND_INT

        # make a structure for the roi timeseries
        roits_row = {'date': img_date,
                     'local_std_time': img_time,
                     'datetime': img_DT,
                     'r_mean': r_mean,
                     'r_std': r_std,
                     'r_5_qtl': r_5,
                     'r_10_qtl': r_10,
                     'r_25_qtl': r_25,
                     'r_50_qtl': r_50,
                     'r_75_qtl': r_75,
                     'r_90_qtl': r_90,
                     'r_95_qtl': r_95,

                     'g_mean': g_mean,
                     'g_std': g_std,
                     'g_5_qtl': g_5,
                     'g_10_qtl': g_10,
                     'g_25_qtl': g_25,
                     'g_50_qtl': g_50,
                     'g_75_qtl': g_75,
                     'g_90_qtl': g_90,
                     'g_95_qtl': g_95,

                     'b_mean': b_mean,
                     'b_std': b_std,
                     'b_5_qtl': b_5,
                     'b_10_qtl': b_10,
                     'b_25_qtl': b_25,
                     'b_50_qtl': b_50,
                     'b_75_qtl': b_75,
                     'b_90_qtl': b_90,
                     'b_95_qtl': b_95,

                     'r_g_correl': rg_cor,
                     'g_b_correl': gb_cor,
                     'b_r_correl': br_cor,

                     'gcc': gcc,
                     'rcc': rcc,
                     'filename': '%s' % (img_file,),
                     'mask_index': mask_index,
                     'solar_elev': sun_elev,
                     'exposure': exposure}

        return roits_row

    def insert_row(self, impath, roimask, mask_index):
        """
        create a ROITimeSeries row dictionary and insert it into
        self.rows list and return the row dictionary.  If there is
        a problem creating the row return None.
        """

        # create row dictionary
        roits_row = self.create_row(impath, roimask, mask_index)
        if not roits_row:
            return None

        # use file name as key, only one row per image file
        rowfn = roits_row['filename']
        FN_INDEX = [row['filename'] for row in self.rows]
        try:
            row_index = FN_INDEX.index(rowfn)
        except:
            row_index = None

        # replace or append
        if row_index:
            self.rows.pop(row_index)
            self.rows.append(roits_row)
        else:
            self.rows.append(roits_row)

        return roits_row

    def append_row(self, impath, roimask, mask_index):
        """
        create a ROITimeSeries row dictionary and append it to
        self.rows list and return the row dictionary.
        """

        # create row dictionary
        roits_row = self.create_row(impath, roimask, mask_index)

        # append row
        if roits_row:
            self.rows.append(roits_row)

        return roits_row

    def format_csvrow(self, roits_row):
        """
        format ROITimeSeries CSV row as string
        """

        row_dt = roits_row['datetime']
        row_doy = row_dt.timetuple().tm_yday

        r_pcts = [roits_row['r_5_qtl'], roits_row['r_10_qtl'],
                  roits_row['r_25_qtl'], roits_row['r_50_qtl'],
                  roits_row['r_75_qtl'], roits_row['r_90_qtl'],
                  roits_row['r_95_qtl']]

        g_pcts = [roits_row['g_5_qtl'], roits_row['g_10_qtl'],
                  roits_row['g_25_qtl'], roits_row['g_50_qtl'],
                  roits_row['g_75_qtl'], roits_row['g_90_qtl'],
                  roits_row['g_95_qtl']]

        b_pcts = [roits_row['b_5_qtl'], roits_row['b_10_qtl'],
                  roits_row['b_25_qtl'], roits_row['b_50_qtl'],
                  roits_row['b_75_qtl'], roits_row['b_90_qtl'],
                  roits_row['b_95_qtl']]

        csvstr_0 = '{},{},{},{},{:.5f},{},{}'
        csvstr_0 = csvstr_0.format(roits_row['date'],
                                   roits_row['local_std_time'],
                                   row_doy,
                                   roits_row['filename'],
                                   roits_row['solar_elev'],
                                   roits_row['exposure'],
                                   roits_row['mask_index'])

        if roits_row['gcc'] == ND_FLOAT:
            csvfmt = '{},{}'
        else:
            csvfmt = '{:.5f},{:.5f}'
        csvstr_1 = csvfmt.format(roits_row['gcc'],
                                 roits_row['rcc'])

        # red
        if roits_row['r_mean'] == ND_FLOAT:
            csvfmt = '{},{}'
        else:
            csvfmt = '{:.5f},{:.5f}'
        csvstr_2 = csvfmt.format(roits_row['r_mean'],
                                 roits_row['r_std'])
        if r_pcts[0] == ND_FLOAT:
            csvfmt = '{}'
        else:
            csvfmt = '{:.0f}'
        csvstr_3 = ','.join(csvfmt.format(k) for k in r_pcts)

        # green
        if roits_row['g_mean'] == ND_FLOAT:
            csvfmt = '{},{}'
        else:
            csvfmt = '{:.5f},{:.5f}'
        csvstr_4 = csvfmt.format(roits_row['g_mean'],
                                 roits_row['g_std'])
        if g_pcts[0] == ND_FLOAT:
            csvfmt = '{}'
        else:
            csvfmt = '{:.0f}'
        csvstr_5 = ','.join(csvfmt.format(k) for k in g_pcts)

        # blue
        if roits_row['b_mean'] == ND_FLOAT:
            csvfmt = '{},{}'
        else:
            csvfmt = '{:.5f},{:.5f}'
        csvstr_6 = csvfmt.format(roits_row['b_mean'],
                                 roits_row['b_std'])
        if b_pcts[0] == ND_FLOAT:
            csvfmt = '{}'
        else:
            csvfmt = '{:.0f}'
        csvstr_7 = ','.join(csvfmt.format(k) for k in b_pcts)

        # csvstr_4 = '{0:.5f},{1:.5f}'.format(roits_row['g_mean'],
        #                                     roits_row['g_std'])
        # csvstr_5 = ','.join('{0:.0f}'.format(k) for k in g_pcts)

        # csvstr_6 = '{0:.5f},{1:.5f}'.format(roits_row['b_mean'],
        #                                     roits_row['b_std'])
        # csvstr_7 = ','.join('{0:.0f}'.format(k) for k in b_pcts)

        if (
            (roits_row['r_g_correl'] == ND_FLOAT) or
            (roits_row['g_b_correl'] == ND_FLOAT) or
            (roits_row['b_r_correl'] == ND_FLOAT)
        ):

            csvfmt = '{},{},{}'
        else:
            csvfmt = '{0:.5f},{1:.5f},{2:.5f}'

        csvstr_8 = csvfmt.format(roits_row['r_g_correl'],
                                 roits_row['g_b_correl'],
                                 roits_row['b_r_correl'])

        csvstr = ','.join([csvstr_0, csvstr_1, csvstr_2, csvstr_3, csvstr_4,
                           csvstr_5, csvstr_6, csvstr_7, csvstr_8])

        return csvstr

    def writeCSV(self, file=""):
        """
        Method for writing an ROITimeSeries to CSV file.  The method
        opens the file for writing.  If no filename is passed
        then write to stdout.
        """
        if file == "":
            fo = sys.stdout
        else:
            fo = open(file, 'w')

        # write header
        hdstrings = []
        hdstrings.append('#\n')
        hdstrings.append('# All-image time series for {0}\n'.format(self.site))
        hdstrings.append('#\n')
        hdstrings.append('# Site: {0}\n'.format(self.site))
        hdstrings.append('# Veg Type: {0}\n'.format(self.roitype))
        hdstrings.append('# ROI ID Number: {0:04d}\n'.format(
            self.sequence_number))
        hdstrings.append('# Lat: {0}\n'.format(self.lat))
        hdstrings.append('# Lon: {0}\n'.format(self.lon))
        hdstrings.append('# Elev: {0}\n'.format(self.elev))
        hdstrings.append('# UTC Offset: {0}\n'.format(self.tzoffset))
        hdstrings.append('# Resize Flag: {0}\n'.format(self.resizeFlg))
        hdstrings.append('# Version: 1\n')
        hdstrings.append('# Creation Date: {0}\n'.
                         format(self.created_at.date()))
        create_time = self.created_at.time()
        hdstrings.append('# Creation Time: {0:02d}:{1:02d}:{2:02d}\n'.
                         format(create_time.hour,
                                create_time.minute,
                                create_time.second))

        # set update date and time
        self.updated_at = datetime.now()
        update_time = self.updated_at.time()
        hdstrings.append('# Update Date: {0}\n'.
                         format(self.updated_at.date()))
        hdstrings.append('# Update Time: {0:02d}:{1:02d}:{2:02d}\n'.
                         format(update_time.hour,
                                update_time.minute,
                                update_time.second))
        hdstrings.append('#\n')
        for line in hdstrings:
            fo.write(line)

        # print fields line
        fields_str = 'date,local_std_time,doy,filename,' + \
                     'solar_elev,exposure,' + \
                     'mask_index,gcc,rcc,' + \
                     'r_mean,r_std,r_5_qtl,r_10_qtl,r_25_qtl,r_50_qtl,' + \
                     'r_75_qtl,r_90_qtl,r_95_qtl,' + \
                     'g_mean,g_std,g_5_qtl,g_10_qtl,g_25_qtl,g_50_qtl,' + \
                     'g_75_qtl,g_90_qtl,g_95_qtl,' + \
                     'b_mean,b_std,b_5_qtl,b_10_qtl,b_25_qtl,b_50_qtl,' + \
                     'b_75_qtl,b_90_qtl,b_95_qtl,' + \
                     'r_g_correl,g_b_correl,b_r_correl\n'
        fo.write(fields_str)

        # sort rows by datetime before writing
        rows = self.rows
        rows_sorted = sorted(rows, key=lambda k: k['datetime'])
        self.rows = rows_sorted

        # print rows in timeseries
        nout = 0
        for row in self.rows:
            rowstr = self.format_csvrow(row)
            fo.write("{0}\n".format(rowstr))
            nout += 1

        # close output
        if not file == "":
            fo.close()

        return nout

    def select_rows(self, tod_min=config.TIME_MIN, tod_max=config.TIME_MAX,
                    sunelev_min=config.MIN_SUN_ANGLE, brt_min=config.MIN_BRT,
                    brt_max=config.MAX_BRT):
        """
        routine to return a list of the rows in self.rows
        which meet the selection criteria for brightness
        and time of day.  Numpy probably has a better (i.e. faster)
        way to do this. Also remove rows where thie image is
        completely black.
        """
        selected_rows = []
        for row in self.rows:
            brt = row['r_mean'] + row['g_mean'] + row['b_mean']
            if (row['datetime'].time() < tod_min) or \
               (row['datetime'].time() > tod_max) or \
               (brt < brt_min) or \
               (brt > brt_max) or \
               (row['solar_elev'] < sunelev_min):
                continue
            else:
                selected_rows.append(row)

        rows = sorted(selected_rows, key=lambda k: k['datetime'])

        return rows

    def readCSV(self, roiTimeSeriesPath):
        """
        Method to read ROITimeSeries object from CSV file and return
        a ROITimeSeries object.  If the comment fields are not present
        they must be set before the object can be written.
        """

        # open file for reading
        f = open(roiTimeSeriesPath, 'r')

        # get comment lines
        comments = list(_get_comments(f))

        # no validation applied to sitename
        site = _get_comment_field(comments, 'Site')
        if site != "":
            self.site = site

        # no validataion applied to ROIListId
        # roilistid = _get_comment_field(comments, 'ROI-id')
        # if roilistid != "":
        #     self.roilistid = roilistid
        roitype = _get_comment_field(comments, 'Veg Type')
        roiseqno = _get_comment_field(comments, 'ROI ID Number')
        self.roitype = roitype
        self.sequence_number = int(roiseqno)
        self.roilistid = "{0}_{1:04d}".format(roitype, int(roiseqno))

        # get Resize Flag if found in header
        resizeflg = _get_comment_field(comments, 'Resize Flag')
        if resizeflg != "":
            # convert to boolean
            if resizeflg == 'True':
                self.resizeFlg = True

        # make sure we can form a proper date time from create_date and
        # create_time
        create_date = _get_comment_field(comments, 'Creation Date')
        create_time = _get_comment_field(comments, 'Creation Time')
        if create_date != "" and create_time != "":
            try:
                (Y, M, D) = create_date.split('-')
                (h, m, s) = create_time.split(':')
                create_dt = datetime(int(Y), int(M), int(D), int(h),
                                     int(m), int(s))
                self.created_at = create_dt
            except ValueError:
                print "Invalid creation date or time in CSV."

        # make sure we can form a proper date time from update_date and
        # update_time
        update_date = _get_comment_field(comments, 'Update Date')
        update_time = _get_comment_field(comments, 'Update Time')
        if update_date != "" and update_time != "":
            try:
                (Y, M, D) = update_date.split('-')
                (h, m, s) = update_time.split(':')
                update_dt = datetime(int(Y), int(M), int(D), int(h),
                                     int(m), int(s))
                self.updated_at = update_dt
            except ValueError:
                print "Invalid update date or time in CSV."

        # do I need to grab lat, lon, elev and tzoffset?  Probably!

        # get timeseries rows
        f.seek(0)
        csvrdr = csv.DictReader(_filter_comments(f))

        roitsRows = []
        for row in csvrdr:

            # turn date and time strings into datetime values
            (im_yr, im_mo, im_dom) = row['date'].split('-')
            (im_hr, im_min, im_sec) = row['local_std_time'].split(':')
            im_dt = datetime(int(im_yr), int(im_mo), int(im_dom),
                             int(im_hr), int(im_min), int(im_sec))

            row['datetime'] = im_dt

            # convert strings to numbers - there's got to be a more
            # efficient way to do this!
            row['solar_elev'] = _float_or_none(row['solar_elev'])
            row['exposure'] = _int_or_none(_float_or_none(row['exposure']))
            row['mask_index'] = _int_or_none(row['mask_index'])
            row['gcc'] = _float_or_none(row['gcc'])
            row['rcc'] = _float_or_none(row['rcc'])
            row['r_mean'] = _float_or_none(row['r_mean'])
            row['r_std'] = _float_or_none(row['r_std'])
            row['r_5_qtl'] = _float_or_none(row['r_5_qtl'])
            row['r_10_qtl'] = _float_or_none(row['r_10_qtl'])
            row['r_25_qtl'] = _float_or_none(row['r_25_qtl'])
            row['r_50_qtl'] = _float_or_none(row['r_50_qtl'])
            row['r_75_qtl'] = _float_or_none(row['r_75_qtl'])
            row['r_90_qtl'] = _float_or_none(row['r_90_qtl'])
            row['r_95_qtl'] = _float_or_none(row['r_95_qtl'])
            row['g_mean'] = _float_or_none(row['g_mean'])
            row['g_std'] = _float_or_none(row['g_std'])
            row['g_5_qtl'] = _float_or_none(row['g_5_qtl'])
            row['g_10_qtl'] = _float_or_none(row['g_10_qtl'])
            row['g_25_qtl'] = _float_or_none(row['g_25_qtl'])
            row['g_50_qtl'] = _float_or_none(row['g_50_qtl'])
            row['g_75_qtl'] = _float_or_none(row['g_75_qtl'])
            row['g_90_qtl'] = _float_or_none(row['g_90_qtl'])
            row['g_95_qtl'] = _float_or_none(row['g_95_qtl'])
            row['b_mean'] = _float_or_none(row['b_mean'])
            row['b_std'] = _float_or_none(row['b_std'])
            row['b_5_qtl'] = _float_or_none(row['b_5_qtl'])
            row['b_10_qtl'] = _float_or_none(row['b_10_qtl'])
            row['b_25_qtl'] = _float_or_none(row['b_25_qtl'])
            row['b_50_qtl'] = _float_or_none(row['b_50_qtl'])
            row['b_75_qtl'] = _float_or_none(row['b_75_qtl'])
            row['b_90_qtl'] = _float_or_none(row['b_90_qtl'])
            row['b_95_qtl'] = _float_or_none(row['b_95_qtl'])
            row['r_g_correl'] = _float_or_none(row['r_g_correl'])
            row['g_b_correl'] = _float_or_none(row['g_b_correl'])
            row['b_r_correl'] = _float_or_none(row['b_r_correl'])
            roitsRows.append(row)

        # do I need to sort here?

        self.rows = roitsRows
        f.close()



# -*- coding: utf-8 -*-

import config
import csv
import re
import sys
from datetime import datetime


def _filter_comments(f):
    """
    filter comments from csv file
    """
    for l in f:
        line = l.rstrip()
        if line and not line.startswith("#"):
            yield line


def _get_comments(f):
    """
    return JUST the comment lines from a csv file
    """
    for l in f:
        line = l.rstrip()
        if line and line.startswith("#"):
            yield line


def _get_comment_field(comments, var_string):
    """
    return value of a field from a list of comment lines

    The comment line with a field has the following format:

    # var_string: var_value

    For example:

    # Creation Date: 2012-02-01

    So we need to create a pattern, find a match and then return
    the value.  No checking of whether the value is valid will
    be done by this routine.
    """

    # set pattern to match
    pattern = '# {0}:\ (?P<var_value>.+)$'.format(var_string)

    var_value = ""
    for line in comments:

        result = re.match(pattern, line)

        if result is not None:
            var_value = result.group('var_value')
            break

    return var_value


class ROIList(object):
    """
    Class for CSV version of ROI List.
    """

    def __init__(self, site='', roitype='', descrip='', sequence_number=0,
                 owner=''):
        """
        create ROIList object which matches database

        Here's a proposed sample ROIList file:

        #
        # ROI List for hubbardbrooknfws
        #
        # Site: hubbardbrooknfws
        # Veg Type: DB
        # ROI ID Number: 0001
        # Owner: mtoomey
        # Creation Date: 2013-02-11
        # Creation Time: 11:19:00
        # Update Date: 2013-02-11
        # Update Time: 11:19:00
        # Description: Full mixed canopy
        #
        start_date,start_time,end_date,end_time,maskfile,sample_image
        2012-09-27,00:00:00,9999-12-31,00:00:00,hubbardbrooknfws_canopy_0001_01.tif,hubbardbrooknfws_2013_02_01_141504.jpg

        """
        self.site = site
        self.roitype = roitype
        self.sequence_number = sequence_number
        self.owner = owner
        self.created_at = datetime.now()
        self.updated_at = datetime.now()
        self.descrip = descrip

        # preform some sanity checks
        if roitype != "":
            try:
                config.ROITypes.index(roitype)
                self.roitype = roitype
            except ValueError:
                sys.stderr.write("Unknown Veg Type in CSV\n")
                raise ValueError

        # sequence number should be decimal
        try:
            self.sequence_number = int(sequence_number)
        except ValueError:
            print "CSV ROI ID Number not valid"

        # initially create the object with an empty list
        # of masks
        self.masks = []

    def namestring(self):
        name = '{0}_{1}_{2:04d}'.format(self.site, self.roitype,
                                        self.sequence_number)
        return name

    def __unicode__(self):
        return self.namestring()

    def length(self):
        return len(self.masks)

    def readCSV(self, roiListPath):
        """
        Method to read in ROIList from a CSV file and return a
        ROIList object.  If the comment fields are not present
        they must be set before the ROIList can be saved. Should
        get comments from database.
        """

        # open file for read-only access with universal line endings
        f = open(roiListPath, 'rU')

        # get comment lines
        comments = list(_get_comments(f))

        # no validation applied to sitename
        site = _get_comment_field(comments, 'Site')
        if site != "":
            self.site = site

        # roitype should be one of types defined in config.py
        roitype = _get_comment_field(comments, 'Veg Type')
        if roitype != "":
            try:
                config.ROITypes.index(roitype)
                self.roitype = roitype
            except ValueError:
                print "Unknown Veg Type in CSV"

        # sequence number should be decimal
        sequence_number = _get_comment_field(comments, 'ROI ID Number')
        if sequence_number != "":
            try:
                self.sequence_number = int(sequence_number)
            except ValueError:
                print "CSV ROI ID Number not valid"

        # make sure we can form a proper date time from create_date and
        # create_time
        create_date = _get_comment_field(comments, 'Creation Date')
        create_time = _get_comment_field(comments, 'Creation Time')
        if create_date != "" and create_time != "":
            try:
                (Y, M, D) = create_date.split('-')
                (h, m, s) = create_time.split(':')
                create_dt = datetime(int(Y), int(M), int(D), int(h),
                                     int(m), int(s))
                self.created_at = create_dt
            except ValueError:
                print "Invalid creation date or time in CSV."

        # make sure we can form a proper date time from update_date and
        # update_time
        update_date = _get_comment_field(comments, 'Update Date')
        update_time = _get_comment_field(comments, 'Update Time')
        if update_date != "" and update_time != "":
            try:
                (Y, M, D) = update_date.split('-')
                (h, m, s) = update_time.split(':')
                update_dt = datetime(int(Y), int(M), int(D), int(h),
                                     int(m), int(s))
                self.updated_at = update_dt
            except ValueError:
                print "Invalid update date or time in CSV."

        # no validation for owner
        self.owner = _get_comment_field(comments, 'Owner')

        # no validation for Description
        self.descrip = _get_comment_field(comments, 'Description')

        # get mask rows
        f.seek(0)
        csvrdr = csv.DictReader(_filter_comments(f))

        roimaskList = []
        for row in csvrdr:

            roi_row = {}

            # turn date and time strings into datetime values
            (start_yr, start_mo, start_dom) = row['start_date'].split('-')
            (start_hr, start_min, start_sec) = row['start_time'].split(':')
            start_dt = datetime(int(start_yr), int(start_mo), int(start_dom),
                                int(start_hr), int(start_min), int(start_sec))

            (end_yr, end_mo, end_dom) = row['end_date'].split('-')
            (end_hr, end_min, end_sec) = row['end_time'].split(':')
            end_dt = datetime(int(end_yr), int(end_mo), int(end_dom),
                              int(end_hr), int(end_min), int(end_sec))

            roi_row['start_dt'] = start_dt
            roi_row['end_dt'] = end_dt
            roi_row['maskfile'] = row['maskfile']
            roi_row['sample_image'] = row['sample_image']

            roimaskList.append(roi_row)

        f.close()

        self.masks = roimaskList

    def writeCSV(self, file=""):
        """
        Method for writing ROIList to CSV file.  The method
        opens the file, filename, for writing.  If no file object is passed
        write to stdout instead.

        Do we need to be careful to avoid overwriting an existing
        file?  Yes!
        """

        if file == "":
            fo = sys.stdout
        else:
            fo = open(file, 'w')

        # Before we do anything, check to make sure the roilist object
        # if valid.  Other tests?  Check for valid user?  Warning/Error if
        # no description?
        errlist = self.checkTimes()
        if len(errlist) > 0:
            sys.stderr.write("Invalid start/end times in ROI list.\n")
            sys.exit(1)

        # write header
        hdstrings = []
        hdstrings.append('#\n')
        hdstrings.append('# ROI List for {0}\n'.format(self.site))
        hdstrings.append('#\n')
        hdstrings.append('# Site: {0}\n'.format(self.site))
        hdstrings.append('# Veg Type: {0}\n'.format(self.roitype))
        hdstrings.append('# ROI ID Number: {0:04d}\n'.format(
            self.sequence_number))
        hdstrings.append('# Owner: {0}\n'.format(self.owner))
        hdstrings.append('# Creation Date: {0}\n'.format(
            self.created_at.date()))
        create_time = self.created_at.time()
        hdstrings.append(
            '# Creation Time: {0:02d}:{1:02d}:{2:02d}\n'.format(
                create_time.hour,
                create_time.minute,
                create_time.second))
        hdstrings.append('# Update Date: {0}\n'.format(
            self.updated_at.date()))

        # set update time to now
        self.updated_at = datetime.now()
        update_time = self.updated_at.time()
        hdstrings.append(
            '# Update Time: {0:02d}:{1:02d}:{2:02d}\n'.format(
                update_time.hour,
                update_time.minute,
                update_time.second))
        hdstrings.append('# Description: {0}\n'.format(self.descrip))
        hdstrings.append('#\n')

        for line in hdstrings:
            fo.write(line)

        # print fields line
        fo.write("start_date,start_time,end_date,end_time,maskfile," +
                 "sample_image\n")

        for mask in self.masks:
            start_date = mask['start_dt'].date()
            start_time = mask['start_dt'].time()
            end_date = mask['end_dt'].date()
            end_time = mask['end_dt'].time()
            fo.write("{0},{1},{2},{3},{4},{5}\n".format(start_date,
                                                        start_time,
                                                        end_date,
                                                        end_time,
                                                        mask['maskfile'],
                                                        mask['sample_image']))
        # close output
        if not file == "":
            fo.close()

    def checkTimes(self):
        """
        make sure that for the mask entry N:

        start_dt(N) >= end_dt(N-1)
        end_dt(N) > start_dt(N)

        """
        error_list = []
        lastmask = {}
        for imask, mask in enumerate(self.masks):
            if mask['end_dt'] <= mask['start_dt']:
                error_list.append({'valid': False,
                                   'mask': imask + 1,
                                   'msg':
                                   'End date-time is before start datetime'})

            if imask > 0:
                if mask['start_dt'] < lastmask['end_dt']:
                    error_list.append({'valid': False,
                                       'mask': imask + 1,
                                       'msg': 'Overlaps previous mask.'})

            lastmask = mask

        return(error_list)



#!/usr/bin/env python

"""
Utility functions for accessing image archive.
"""

import os
import re
import sys
from datetime import datetime
from datetime import timedelta

import ephem
import requests

from . import config

# ####################################################################


def fn2date(sitename, filename, irFlag=False):
    """
    Function to extract the date from a "standard" filename based on a
    sitename.  Here we assume the filename format is the standard:

          sitename_YYYY_MM_DD_HHNNSS.jpg

    So we just grab components from fixed positions.  If irFlag is
    True then the "standard" format is:

          sitename_IR_YYYY_MM_DD_HHNNSS.jpg

    """

    if irFlag:
        prefix = sitename + "_IR"
    else:
        prefix = sitename

    # set start of datetime part of name
    nstart = len(prefix) + 1

    # assume 3-letter extension e.g. ".jpg"
    dtstring = filename[nstart:-4]

    # extract date-time pieces
    year = int(dtstring[0:4])
    mon = int(dtstring[5:7])
    day = int(dtstring[8:10])
    hour = int(dtstring[11:13])
    mins = int(dtstring[13:15])
    sec = int(dtstring[15:17])

    # return list
    return [year, mon, day, hour, mins, sec]

# ####################################################################


def fn2datetime(sitename, filename, irFlag=False):
    """
    Function to extract the date from a "standard" filename based on a
    sitename.  Here we assume the filename format is the standard:

          sitename_YYYY_MM_DD_HHNNSS.jpg

    So we just grab components from fixed positions.  If irFlag is
    True then the "standard" format is:

          sitename_IR_YYYY_MM_DD_HHNNSS.jpg

    """

    if irFlag:
        prefix = sitename + "_IR"
    else:
        prefix = sitename

    # set start of datetime part of name
    nstart = len(prefix) + 1

    # assume 3-letter extension e.g. ".jpg"
    dtstring = filename[nstart:-4]

    # extract date-time pieces
    year = int(dtstring[0:4])
    mon = int(dtstring[5:7])
    day = int(dtstring[8:10])
    hour = int(dtstring[11:13])
    mins = int(dtstring[13:15])
    sec = int(dtstring[15:17])

    # return list
    return datetime(year, mon, day, hour, mins, sec)

# ####################################################################


def getsiteimglist(sitename,
                   startDT=datetime(1990, 1, 1, 0, 0, 0),
                   endDT=datetime.now(),
                   getIR=False):
    """
    Returns a list of imagepath names for ALL images in
    archive for specified site.  Optional arguments:
      getIR   : If set to true only return IR images.
      startDT : Start datetime for image list
      endDT   : End datetime for image list

    NOTE: This might be lots faster if we just do a glob.glob()
    on a pattern.  Might not be quite as robust since we're skipping
    the check the .jpg file being a regular file.  See, getImageCount()
    below for how this would work!
    """

    # get startyear and endyear
    startYear = startDT.year
    endYear = endDT.year

    # get startmonth and endmonth
    startMonth = startDT.month
    endMonth = endDT.month

    imglist = []
    sitepath = os.path.join(config.archive_dir, sitename)
    if not os.path.exists(sitepath):
        return imglist

    # get a list of files in the directory
    yeardirs = os.listdir(sitepath)

    # loop over all files
    for yeardir in yeardirs:

        # check that its a directory
        yearpath = os.path.join(sitepath, yeardir)
        if not os.path.isdir(yearpath):
            continue

        # check if this yeardir could be a 4-digit year.  if not skip
        if not re.match('^\d\d\d\d$', yeardir):
            continue

        # check if we're before startYear
        if (int(yeardir) < startYear) | (int(yeardir) > endYear):
            continue

        # get a list of all files in year directory
        mondirs = os.listdir(yearpath)

        # loop over all files
        for mondir in mondirs:

            # check that its a directory
            monpath = os.path.join(yearpath, mondir)
            if not os.path.isdir(monpath):
                continue

            # check if this mondir could be a 2-digit month.  if not skip
            if not re.match('^\d\d$', mondir):
                continue

            # check month range
            if (int(mondir) < 1) | (int(mondir) > 12):
                continue

            # check start year/month
            if (int(yeardir) == startYear) & (int(mondir) < startMonth):
                continue

            # check end year/month
            if (int(yeardir) == endYear) & (int(mondir) > endMonth):
                continue

            try:
                imgfiles = os.listdir(monpath)
                if getIR:
                    image_re = "^%s_IR_%s_%s_.*\.jpg$" % (
                        sitename, yeardir, mondir)
                else:
                    image_re = "^%s_%s_%s_.*\.jpg$" % (
                        sitename, yeardir, mondir)

                for imgfile in imgfiles:
                    # check for pattern match
                    if not re.match(image_re, imgfile):
                        continue

                    # get image time
                    [yr, mo, md, hr, mn, sc] = fn2date(
                        sitename, imgfile, irFlag=getIR)
                    img_dt = datetime(yr, mo, md, hr, mn, sc)

                    if img_dt < startDT:
                        continue

                    if img_dt > endDT:
                        continue

                    # only add regular files
                    imgpath = os.path.join(monpath, imgfile)
                    if not os.path.isdir(imgpath):
                        imglist.append(imgpath)

            except OSError, e:
                if e.errno == 20:
                    continue
                else:
                    errstring = "Python OSError: %s" % (e,)
                    print errstring

    imglist.sort()
    return imglist

# ####################################################################


def getsiteinfo(sitename):
    """
    Simple function to return the site info for a single site in a dictionary
    by grabbing JSON from a URL.
    """

    siteinfo = {}
    try:
        infourl = "https://phenocam.sr.unh.edu/webcam/" + \
                  "sites/{0}/info/".format(sitename)
        response = requests.get(infourl)
    except:
        sys.stderr.write("Error getting site info.\n")
        return None

    siteinfo = response.json()
    return siteinfo

# ####################################################################


def deg2dms(angle):
    """
    helper function to convert angle in fractional
    degrees to a string of the form dd:mm:ss
    """
    deg = int(angle)
    degdiff = abs(angle - float(deg))
    minangle = degdiff * 60.
    min = int(minangle)
    mindiff = minangle - float(min)
    sec = int(mindiff * 60.)
    dmsstr = "{0:02d}:{1:02d}:{2:02d}".format(deg, min, sec)
    return dmsstr

# ####################################################################


def dms2deg(dmsstr):
    """
    convert dms string in form 'dd:mm:ss' to fractional degrees.
    """
    (dd, mm, ss) = dmsstr.split(':')
    degrees = dd + mm * 1. / 60. + ss * 1. / 3600.
    return degrees

# ####################################################################


def sunelev(lat, lon, dt, tzoffset):
    """
    function to return the solar elevation at a given
    latitude (decimal degrees) and longitude (decimal degrees)
    and local date and time.  The time is assumed to be
    the local "standard" time and the offset is the rawOffset
    from UTC/GMT (i.e. the offset for "standard time").
    """

    # set up observer for ephem package
    site = ephem.Observer()
    site.lat = deg2dms(lat)
    site.lon = deg2dms(lon)

    # apply timezone offset to local time to give
    # utc time
    utcdt = dt + timedelta(hours=-tzoffset)
    site.date = utcdt

    # get sun position
    sun = ephem.Sun(site)

    # return elevation is decimal degrees
    elev = sun.alt / ephem.degree

    return elev



# -*- coding: utf-8 -*-

import config
import csv
import re
import sys
from datetime import date
from datetime import datetime
from datetime import time

from . import utils

ND_STRING = config.ND_STRING
ND_FLOAT = config.ND_FLOAT
ND_INT = config.ND_INT


def _filter_comments(f):
    """
    filter comments from csv file
    """
    for l in f:
        line = l.rstrip()
        if line and not line.startswith("#"):
            yield line


def _get_comments(f):
    """
    return JUST the comment lines from a csv file
    """
    for l in f:
        line = l.rstrip()
        if line and line.startswith("#"):
            yield line


def _get_comment_field(comments, var_string):
    """
    return value of a field from a list of comment lines

    The comment line with a field has the following format:

    # var_string: var_value

    For example:

    # Creation Date: 2012-02-01

    So we need to create a pattern, find a match and then return
    the value.  No checking of whether the value is valid will
    be done by this routine.
    """

    # set pattern to match
    pattern = '# {0}:\ (?P<var_value>.+)$'.format(var_string)

    var_value = ""
    for line in comments:

        result = re.match(pattern, line)

        if result is not None:
            var_value = result.group('var_value')
            break

    return var_value


def _float_or_none(str):
    """
    Try to convert string to float. Return float or None value if a
    ValueError exception is raised.
    """
    try:
        retval = float(str)
    except ValueError:
        retval = ND_FLOAT

    if retval == -9999.:
        retval = ND_FLOAT

    return retval


def _int_or_none(str):
    """
    Try to convert string to integer. Return int or None value
    if ValueError is raised.
    """
    try:
        retval = int(str)
    except ValueError:
        retval = ND_INT

    if retval == -9999:
        retval = ND_INT

    return retval


class GCCTimeSeries(object):
    """
    Class for CSV version of GCC Timeseries.  There is currently
    no equivalent dB table for the GCC time series object.

    Here's a proposed sample GCCTimeSeries CSV file format:

        #
        # 1-day summary product file for harvardbarn
        #
        # Site: harvardbarn
        # Veg Type: DB
        # ROI ID Number: 0001
        # Lat:
        # Lon:
        # Elev:
        # UTC Offset:
        # Image Count Threshold: 1
        # Aggregation Period: 1
        # Time of Day Min: 00:00:00
        # Time of Day Max: 23:59:59
        # Creation Date: 2013-03-29
        # Creation Time: 12:10:59
        # Update Date: 2013-03-29
        # Update Time: 14:16:43
        # Solar Elevation Min: 5.0
        # ROI Brightness Min: 130
        # ROI Brightness Max: 700
        #
        date,year,doy,image_count,midday_filename,midday_r,midday_g,
        midday_b,midday_gcc,midday_rcc,r_mean,r_std,g_mean,g_std,
        b_mean,g_std,gcc_mean,gcc_std,gcc_50,gcc_75,gcc_90,
        rcc_mean,rcc_std,rcc_50,rcc_75,rcc_90, max_solar_elev,
        snow_flag,outlierflag_gcc_mean,
        outlierflag_gcc_50,outlierflag_gcc_75,outlierflag_gcc_90

    """

    def __init__(self, site='', ROIListID='', nday=1,
                 nmin=config.NIMAGE_MIN,
                 brt_min=config.MIN_BRT,
                 brt_max=config.MAX_BRT,
                 tod_min=time(0, 0, 0),
                 tod_max=time(23, 59, 59),
                 sunelev_min=config.MIN_SUN_ANGLE):
        """ create GCCTimeSeries object """

        self.site = site
        self.roilistid = ROIListID
        self.nmin = nmin
        self.tod_min = tod_min
        self.tod_max = tod_max
        self.nday = nday
        self.created_at = datetime.now()
        self.updated_at = datetime.now()
        self.sunelev_min = sunelev_min
        self.brt_min = brt_min
        self.brt_max = brt_max
        self.rows = []

        # get roitype and sequence_number from roilistid
        roitype, roiseqno = self.roilistid.split('_')
        self.roitype = roitype
        self.sequence_number = roiseqno

        if site:
            try:
                si = utils.getsiteinfo(site)
            except:
                si = {'lat': None,
                      'lon': None,
                      'elev': None,
                      'tzoffset': None}
        else:
            si = {'lat': None,
                  'lon': None,
                  'elev': None,
                  'tzoffset': None}

        self.lat = si['lat']
        self.lon = si['lon']
        self.elev = si['elev']
        self.tzoffset = si['tzoffset']

    def readCSV(self, gccTimeSeriesPath):
        """
        Method to read GCCTimeSeries object from CSV file and return
        a GCCTimeSeries object.
        """

        # open file for reading
        f = open(gccTimeSeriesPath, 'r')

        # get comment lines
        comments = list(_get_comments(f))

        # no validation applied to sitename
        site = _get_comment_field(comments, 'Site')
        if site != "":
            self.site = site

        # no validataion applied to ROIListId
        # roilistid = _get_comment_field(comments, 'ROI-id')
        # if roilistid != "":
        #     self.roilistid = roilistid
        roitype = _get_comment_field(comments, 'Veg Type')
        roiseqno = _get_comment_field(comments, 'ROI ID Number')
        roilistid = roitype + "_" + roiseqno
        self.roilistid = roilistid
        self.roitype = roitype
        self.sequence_number = roiseqno

        # probably need to read in lat, lon, elev, tz_offset
        # really this should be a "site" class!!!  If you
        # use vegindex.get_gcc_timeseries it should work.

        # Image Count Threshold: 1
        nmin = _get_comment_field(comments, 'Image Count Threshold')
        if nmin != "":
            self.nmin = int(nmin)

        # Aggregation Period: 1
        nday = _get_comment_field(comments, 'Aggregation Period')
        if nday != "":
            self.nday = int(nday)

        # Time of Day Min: 00:00:00
        time_str = _get_comment_field(comments, 'Time of Day Min')
        if time_str != "":
            [hr_str, min_str, sec_str] = time_str.split(':')
            self.tod_min = time(int(hr_str), int(min_str), int(sec_str))

        # Time of Day Max: 23:59:59
        time_str = _get_comment_field(comments, 'Time of Day Max')
        if time_str != "":
            [hr_str, min_str, sec_str] = time_str.split(':')
            self.tod_max = time(int(hr_str), int(min_str), int(sec_str))

        # Solar Elevation Min: 5.0
        sunelev_min_str = _get_comment_field(comments,
                                             'Solar Elevation Min')
        if sunelev_min_str != "":
            self.sunelev_min = float(sunelev_min_str)

        # ROI Brightness Min: 130
        brt_min_str = _get_comment_field(comments, 'ROI Brightness Min')
        if brt_min_str != "":
            self.brt_min = int(brt_min_str)

        # ROI Brightness Max: 700
        brt_max_str = _get_comment_field(comments, 'ROI Brightness Max')
        if brt_max_str != "":
            self.brt_max = int(brt_max_str)

        # make sure we can form a proper date time from create_date and
        # create_time
        create_date = _get_comment_field(comments, 'Creation Date')
        create_time = _get_comment_field(comments, 'Creation Time')
        if create_date != "" and create_time != "":
            try:
                (Y, M, D) = create_date.split('-')
                (h, m, s) = create_time.split(':')
                create_dt = datetime(int(Y), int(M), int(D), int(h),
                                     int(m), int(s))
                self.created_at = create_dt
            except ValueError:
                print "Invalid creation date or time in CSV."

        # make sure we can form a proper date time from update_date and
        # update_time
        update_date = _get_comment_field(comments, 'Update Date')
        update_time = _get_comment_field(comments, 'Update Time')
        if update_date != "" and update_time != "":
            try:
                (Y, M, D) = update_date.split('-')
                (h, m, s) = update_time.split(':')
                update_dt = datetime(int(Y), int(M), int(D), int(h),
                                     int(m), int(s))
                self.updated_at = update_dt
            except ValueError:
                print "Invalid update date or time in CSV."

        # get timeseries rows
        f.seek(0)
        csvrdr = csv.DictReader(_filter_comments(f))

        gcctsRows = []
        for row in csvrdr:

            # convert date string to date object
            [yr_str, mo_str, dom_str] = row['date'].split('-')
            gcc_date = date(int(yr_str), int(mo_str), int(dom_str))
            row['date'] = gcc_date
            row['year'] = gcc_date.year

            # turn strings to numbers checking for empty/missing and
            # no-data values
            row['image_count'] = int(row['image_count'])

            if row['image_count'] == 0:
                row['doy'] = _int_or_none(row['doy'])
                row['midday_filename'] = ND_STRING
                row['midday_r'] = ND_FLOAT
                row['midday_g'] = ND_FLOAT
                row['midday_b'] = ND_FLOAT
                row['midday_gcc'] = ND_FLOAT
                row['midday_rcc'] = ND_FLOAT
                row['r_mean'] = ND_FLOAT
                row['r_std'] = ND_FLOAT
                row['g_mean'] = ND_FLOAT
                row['g_std'] = ND_FLOAT
                row['b_mean'] = ND_FLOAT
                row['b_std'] = ND_FLOAT
                row['gcc_mean'] = ND_FLOAT
                row['gcc_std'] = ND_FLOAT
                row['gcc_50'] = ND_FLOAT
                row['gcc_75'] = ND_FLOAT
                row['gcc_90'] = ND_FLOAT
                row['rcc_mean'] = ND_FLOAT
                row['rcc_std'] = ND_FLOAT
                row['rcc_50'] = ND_FLOAT
                row['rcc_75'] = ND_FLOAT
                row['rcc_90'] = ND_FLOAT
                row['max_solar_elev'] = ND_FLOAT
                row['snow_flag'] = ND_INT
                row['outlierflag_gcc_mean'] = ND_INT
                row['outlierflag_gcc_50'] = ND_INT
                row['outlierflag_gcc_75'] = ND_INT
                row['outlierflag_gcc_90'] = ND_INT

            elif row['image_count'] < self.nmin:
                row['doy'] = _int_or_none(row['doy'])
                if not row['midday_filename']:
                    row['midday_filename'] = ND_STRING
                row['midday_r'] = _float_or_none(row['midday_r'])
                row['midday_g'] = _float_or_none(row['midday_g'])
                row['midday_b'] = _float_or_none(row['midday_b'])
                row['midday_gcc'] = _float_or_none(row['midday_gcc'])
                row['midday_rcc'] = _float_or_none(row['midday_rcc'])
                row['r_mean'] = ND_FLOAT
                row['r_std'] = ND_FLOAT
                row['g_mean'] = ND_FLOAT
                row['g_std'] = ND_FLOAT
                row['b_mean'] = ND_FLOAT
                row['b_std'] = ND_FLOAT
                row['gcc_mean'] = ND_FLOAT
                row['gcc_std'] = ND_FLOAT
                row['gcc_50'] = ND_FLOAT
                row['gcc_75'] = ND_FLOAT
                row['gcc_90'] = ND_FLOAT
                row['rcc_mean'] = ND_FLOAT
                row['rcc_std'] = ND_FLOAT
                row['rcc_50'] = ND_FLOAT
                row['rcc_75'] = ND_FLOAT
                row['rcc_90'] = ND_FLOAT
                row['max_solar_elev'] = ND_FLOAT
                row['snow_flag'] = ND_INT
                row['outlierflag_gcc_mean'] = ND_INT
                row['outlierflag_gcc_50'] = ND_INT
                row['outlierflag_gcc_75'] = ND_INT
                row['outlierflag_gcc_90'] = ND_INT

            else:
                row['doy'] = _int_or_none(row['doy'])
                row['midday_r'] = _float_or_none(row['midday_r'])
                row['midday_g'] = _float_or_none(row['midday_g'])
                row['midday_b'] = _float_or_none(row['midday_b'])
                row['midday_gcc'] = _float_or_none(row['midday_gcc'])
                row['midday_rcc'] = _float_or_none(row['midday_rcc'])
                row['r_mean'] = _float_or_none(row['r_mean'])
                row['r_std'] = _float_or_none(row['r_std'])
                row['g_mean'] = _float_or_none(row['g_mean'])
                row['g_std'] = _float_or_none(row['g_std'])
                row['b_mean'] = _float_or_none(row['b_mean'])
                row['b_std'] = _float_or_none(row['b_std'])
                row['gcc_mean'] = _float_or_none(row['gcc_mean'])
                row['gcc_std'] = _float_or_none(row['gcc_std'])
                row['gcc_50'] = _float_or_none(row['gcc_50'])
                row['gcc_75'] = _float_or_none(row['gcc_75'])
                row['gcc_90'] = _float_or_none(row['gcc_90'])
                row['rcc_mean'] = _float_or_none(row['rcc_mean'])
                row['rcc_std'] = _float_or_none(row['rcc_std'])
                row['rcc_50'] = _float_or_none(row['rcc_50'])
                row['rcc_75'] = _float_or_none(row['rcc_75'])
                row['rcc_90'] = _float_or_none(row['rcc_90'])
                row['max_solar_elev'] = _float_or_none(row['max_solar_elev'])
                row['snow_flag'] = _int_or_none(row['snow_flag'])
                row['outlierflag_gcc_mean'] = _int_or_none(
                    row['outlierflag_gcc_mean'])
                row['outlierflag_gcc_50'] = _int_or_none(
                    row['outlierflag_gcc_50'])
                row['outlierflag_gcc_75'] = _int_or_none(
                    row['outlierflag_gcc_75'])
                row['outlierflag_gcc_90'] = _int_or_none(
                    row['outlierflag_gcc_90'])

            gcctsRows.append(row)

        self.rows = gcctsRows

    def insert_row(self, date, doy, image_count,
                   midday_filename,
                   midday_r,
                   midday_g,
                   midday_b,
                   midday_gcc,
                   midday_rcc,
                   r_mean, r_std,
                   g_mean, g_std,
                   b_mean, b_std,
                   gcc_mean, gcc_std,
                   gcc_50, gcc_75, gcc_90,
                   rcc_mean, rcc_std,
                   rcc_50, rcc_75, rcc_90,
                   max_solar_elev, snow_flag,
                   outlierflag_gcc_mean,
                   outlierflag_gcc_50,
                   outlierflag_gcc_75,
                   outlierflag_gcc_90):
        """
        create a GCCTimeSeries row dictionary for a particluar date
        insert it into self.rows list and return the row dictionary.
        """

        # create a row dictionary
        year = date.year
        gccts_row = {'date': date,
                     'year': year,
                     'doy': doy,
                     'image_count': image_count,
                     'midday_filename': midday_filename,
                     'midday_r': midday_r,
                     'midday_g': midday_g,
                     'midday_b': midday_b,
                     'midday_gcc': midday_gcc,
                     'midday_rcc': midday_rcc,
                     'r_mean': r_mean,
                     'r_std': r_std,
                     'g_mean': g_mean,
                     'g_std': g_std,
                     'b_mean': b_mean,
                     'b_std': b_std,
                     'gcc_mean': gcc_mean,
                     'gcc_std': gcc_std,
                     'gcc_50': gcc_50,
                     'gcc_75': gcc_75,
                     'gcc_90': gcc_90,
                     'rcc_mean': rcc_mean,
                     'rcc_std': rcc_std,
                     'rcc_50': rcc_50,
                     'rcc_75': rcc_75,
                     'rcc_90': rcc_90,
                     'max_solar_elev': max_solar_elev,
                     'snow_flag': snow_flag,
                     'outlierflag_gcc_mean': outlierflag_gcc_mean,
                     'outlierflag_gcc_50': outlierflag_gcc_50,
                     'outlierflag_gcc_75': outlierflag_gcc_75,
                     'outlierflag_gcc_90': outlierflag_gcc_90}

        # use date as key
        DT_INDEX = [row['date'] for row in self.rows]
        try:
            row_index = DT_INDEX.index(date)
        except:
            row_index = None

        # replace or append
        if row_index:
            self.rows.pop(row_index)
            self.rows.append(gccts_row)
        else:
            self.rows.append(gccts_row)

        return gccts_row

    def format_csvrow(self, gccts_row):
        """
        Format GCCTimeSeries CSV row as string.  To handle no-data/missing
        values using strings like 'NA' or 'NULL' we use different
        formats depending on whether there is missing data or not.
        The problem is that a fixed format field for real numbers
        {:.5f} doesn't handle a string like NA.

        """

        if gccts_row['image_count'] >= self.nmin:
            line_format = '{0},{1},{2},{3},{4},' + \
                          '{5:.5f},{6:.5f},{7:.5f},{8:.5f},{9:.5f},' + \
                          '{10:.5f},{11:.5f},{12:.5f}, {13:.5f},{14:.5f},' + \
                          '{15:.5f},{16:.5f},{17:.5f},{18:.5f},{19:.5f},' + \
                          '{20:.5f},{21:.5f},{22:.5f},{23:.5f},{24:.5f},' + \
                          '{25:.5f},{26:.5f},{27},{28},{29},' + \
                          '{30},{31}'

        # case where there are some images but less than the threshold to
        # print a aggregate value
        elif ((gccts_row['image_count'] < self.nmin) and
              (gccts_row['image_count'] > 0)):
            line_format = '{0},{1},{2},{3},{4},' + \
                          '{5:.5f},{6:.5f},{7:.5f},{8:.5f},{9:.5f},' + \
                          '{10},{11},{12},{13},{14},' + \
                          '{15},{16},{17},{18},{19},' + \
                          '{20},{21},{22},{23},{24},' + \
                          '{25},{26},{27},{28},{29},' + \
                          '{30},{31}'

        # case where there are no images for this time period
        else:
            line_format = '{0},{1},{2},{3},{4},' + \
                          '{5},{6},{7},{8},{9},' + \
                          '{10},{11},{12},{13},{14},' + \
                          '{15},{16},{17},{18},{19},' + \
                          '{20},{21},{22},{23},{24},' + \
                          '{25},{26},{27},{28},{29},' + \
                          '{30},{31}'

        csvstr = line_format.format(
            gccts_row['date'],
            gccts_row['year'],
            gccts_row['doy'],
            gccts_row['image_count'],
            gccts_row['midday_filename'],
            gccts_row['midday_r'],
            gccts_row['midday_g'],
            gccts_row['midday_b'],
            gccts_row['midday_gcc'],
            gccts_row['midday_rcc'],
            gccts_row['r_mean'],
            gccts_row['r_std'],
            gccts_row['g_mean'],
            gccts_row['g_std'],
            gccts_row['b_mean'],
            gccts_row['b_std'],
            gccts_row['gcc_mean'],
            gccts_row['gcc_std'],
            gccts_row['gcc_50'],
            gccts_row['gcc_75'],
            gccts_row['gcc_90'],
            gccts_row['rcc_mean'],
            gccts_row['rcc_std'],
            gccts_row['rcc_50'],
            gccts_row['rcc_75'],
            gccts_row['rcc_90'],
            gccts_row['max_solar_elev'],
            gccts_row['snow_flag'],
            gccts_row['outlierflag_gcc_mean'],
            gccts_row['outlierflag_gcc_50'],
            gccts_row['outlierflag_gcc_75'],
            gccts_row['outlierflag_gcc_90'])

        return csvstr

    def writeCSV(self, file=""):
        """
        Method for writing GCCTimeSeries to CSV file.  The method opens
        the file, file, for writing.  If no file object is passed
        write to standard out.

        Do we need to be careful to avoid overwriting files?
        """

        if file == "":
            fo = sys.stdout
        else:
            fo = open(file, 'w')

        # write header
        hdstrings = []
        hdstrings.append('#\n')
        hdstrings.append('# {0}-day summary product time' +
                         'series for {1}\n'.format(self.nday, self.site))
        hdstrings.append('#\n')
        hdstrings.append('# Site: {0}\n'.format(self.site))
        hdstrings.append('# Veg Type: {0}\n'.format(self.roitype))
        hdstrings.append('# ROI ID Number: {0}\n'.format(
            self.sequence_number))
        hdstrings.append('# Lat: {0}\n'.format(self.lat))
        hdstrings.append('# Lon: {0}\n'.format(self.lon))
        hdstrings.append('# Elev: {0}\n'.format(self.elev))
        hdstrings.append('# UTC Offset: {0}\n'.format(self.tzoffset))
        hdstrings.append('# Image Count Threshold: {0}\n'.format(self.nmin))
        hdstrings.append('# Aggregation Period: {0}\n'.format(self.nday))
        hdstrings.append('# Solar Elevation Min: {0}\n'.format(
            self.sunelev_min))
        hdstrings.append('# Time of Day Min: {0}\n'.format(self.tod_min))
        hdstrings.append('# Time of Day Max: {0}\n'.format(self.tod_max))
        hdstrings.append('# ROI Brightness Min: {0}\n'.format(self.brt_min))
        hdstrings.append('# ROI Brightness Max: {0}\n'.format(self.brt_max))
        hdstrings.append('# Creation Date: {0}\n'.format(
            self.created_at.date()))
        create_time = self.created_at.time()
        hdstrings.append('# Creation Time: {0:02d}:{1:02d}:' +
                         '{2:02d}\n'.format(create_time.hour,
                                            create_time.minute,
                                            create_time.second))
        # set update time to now
        self.updated_at = datetime.now()
        update_time = self.updated_at.time()
        hdstrings.append('# Update Date: {0}\n'.format(
            self.updated_at.date()))
        hdstrings.append('# Update Time: {0:02d}:{1:02d}:' +
                         '{2:02d}\n'.format(update_time.hour,
                                            update_time.minute,
                                            update_time.second))
        hdstrings.append('#\n')

        for line in hdstrings:
            fo.write(line)

        # sort rows by datetime before writing
        rows = self.rows
        rows_sorted = sorted(rows, key=lambda k: k['date'])
        self.rows = rows_sorted

        # set up fieldnames - this should be in a global!
        gccts_fn = [
            'date',
            'year',
            'doy',
            'image_count',
            'midday_filename',
            'midday_r',
            'midday_g',
            'midday_b',
            'midday_gcc',
            'midday_rcc',
            'r_mean',
            'r_std',
            'g_mean',
            'g_std',
            'b_mean',
            'b_std',
            'gcc_mean',
            'gcc_std',
            'gcc_50',
            'gcc_75',
            'gcc_90',
            'rcc_mean',
            'rcc_std',
            'rcc_50',
            'rcc_75',
            'rcc_90',
            'max_solar_elev',
            'snow_flag',
            'outlierflag_gcc_mean',
            'outlierflag_gcc_50',
            'outlierflag_gcc_75',
            'outlierflag_gcc_90'
        ]

        #
        # create csvwriter
        csvwriter = csv.DictWriter(fo, delimiter=',',
                                   fieldnames=gccts_fn,
                                   lineterminator='\n',
                                   quoting=csv.QUOTE_NONE)

        # write header
        csvwriter.writerow(dict((fn, fn) for fn in gccts_fn))
        # csvwriter.writeheader()

        # write gcc ts data
        for row in self.rows:

            # format row
            formatted_row = row
            if row['image_count'] >= self.nmin:
                formatted_row['midday_r'] = '{0:.5f}'.format(row['midday_r'])
                formatted_row['midday_g'] = '{0:.5f}'.format(row['midday_g'])
                formatted_row['midday_b'] = '{0:.5f}'.format(row['midday_b'])
                formatted_row['midday_gcc'] = '{0:.5f}'.format(
                    row['midday_gcc'])
                formatted_row['midday_rcc'] = '{0:.5f}'.format(
                    row['midday_rcc'])
                formatted_row['r_mean'] = '{0:.5f}'.format(row['r_mean'])
                formatted_row['r_std'] = '{0:.5f}'.format(row['r_std'])
                formatted_row['g_mean'] = '{0:.5f}'.format(row['g_mean'])
                formatted_row['g_std'] = '{0:.5f}'.format(row['g_std'])
                formatted_row['b_mean'] = '{0:.5f}'.format(row['b_mean'])
                formatted_row['b_std'] = '{0:.5f}'.format(row['b_std'])
                formatted_row['gcc_mean'] = '{0:.5f}'.format(row['gcc_mean'])
                formatted_row['gcc_std'] = '{0:.5f}'.format(row['gcc_std'])
                formatted_row['gcc_50'] = '{0:.5f}'.format(row['gcc_50'])
                formatted_row['gcc_75'] = '{0:.5f}'.format(row['gcc_75'])
                formatted_row['gcc_90'] = '{0:.5f}'.format(row['gcc_90'])
                formatted_row['rcc_mean'] = '{0:.5f}'.format(row['rcc_mean'])
                formatted_row['rcc_std'] = '{0:.5f}'.format(row['rcc_std'])
                formatted_row['rcc_50'] = '{0:.5f}'.format(row['rcc_50'])
                formatted_row['rcc_75'] = '{0:.5f}'.format(row['rcc_75'])
                formatted_row['rcc_90'] = '{0:.5f}'.format(row['rcc_90'])
                formatted_row['max_solar_elev'] = '{0:.5f}'.format(
                    row['max_solar_elev'])
            csvwriter.writerow(formatted_row)

        # close file
        if not file == "":
            fo.close()

        # return number of rows
        return len(self.rows)



# -*- coding: utf-8 -*-

"""
test_roitimeseries
------------------

Tests for `vegindex.roitimeseries` module.
"""

import os

import numpy as np
from PIL import Image
from pkg_resources import Requirement
from pkg_resources import resource_filename

from vegindex import config
from vegindex import roitimeseries

SAMPLE_DATA_DIR = os.path.join(os.path.dirname(__file__),
                               'sample_data')


def test_roits_dnmeans():
    """
    test calculating DN means from image/mask pair
    """

    image_file = 'harvard_2009_06_30_120138.jpg'
    mask_file = 'harvard_DB_0001_01.tif'
    sitename, year, month, dom, xx = image_file.split('_')

    image_path = os.path.join(SAMPLE_DATA_DIR, sitename, year,
                              month, image_file)
    mask_path = os.path.join(SAMPLE_DATA_DIR, sitename, 'ROI',
                             mask_file)

    im = Image.open(image_path)
    roimask = Image.open(mask_path)

    r_mean, g_mean, b_mean, brt = roitimeseries.get_dn_means(im,
                                                             roimask)

    np.testing.assert_approx_equal(r_mean, 48.82500, 3)
    np.testing.assert_approx_equal(g_mean, 69.22912, 3)
    np.testing.assert_approx_equal(b_mean, 42.85413, 3)
    np.testing.assert_approx_equal(brt, 310.30830, 3)


def test_roits_roistats():
    """
    test calculating ROI stats from image/mask pair
    """

    image_file = 'harvard_2009_06_30_120138.jpg'
    mask_file = 'harvard_DB_0001_01.tif'
    sitename, year, month, dom, xx = image_file.split('_')

    image_path = os.path.join(SAMPLE_DATA_DIR, sitename, year,
                              month, image_file)
    mask_path = os.path.join(SAMPLE_DATA_DIR, sitename, 'ROI',
                             mask_file)

    im = Image.open(image_path)
    roimask = Image.open(mask_path)

    [r_stats, g_stats, b_stats, RG_cor, GB_cor,
     BR_cor] = roitimeseries.get_roi_stats(im, roimask)

    r_mean = r_stats['mean']
    r_sd = r_stats['stdev']
    r_pcts = r_stats['percentiles']
    print r_pcts
    g_mean = g_stats['mean']
    g_sd = g_stats['stdev']
    g_pcts = g_stats['percentiles']
    b_mean = b_stats['mean']
    b_sd = b_stats['stdev']
    b_pcts = b_stats['percentiles']

    np.testing.assert_approx_equal(r_mean, 48.82500, 3)
    np.testing.assert_approx_equal(r_sd, 29.08593, 3)
    np.testing.assert_equal(r_pcts[0], 9)
    np.testing.assert_equal(r_pcts[6], 102)
    np.testing.assert_approx_equal(g_mean, 69.22912, 3)
    np.testing.assert_approx_equal(g_sd, 32.44997, 3)
    np.testing.assert_approx_equal(b_mean, 42.85413, 3)
    np.testing.assert_approx_equal(b_sd, 27.45573, 3)
    np.testing.assert_approx_equal(RG_cor, 0.975, 3)
    np.testing.assert_approx_equal(GB_cor, 0.944, 3)
    np.testing.assert_approx_equal(BR_cor, 0.964, 3)



# -*- coding: utf-8 -*-

"""
Script to generate a csv with summary files from a roi
timeseries csv file.  Output format will be:

date,year,doy,image_count,midday_filename,midday_r,midday_g,midday_b,midday_gcc,midday_rcc,
r_mean,r_std,g_mean,g_std,b_mean,b_std,gcc_mean,gcc_std,gcc_50,gcc_75,gcc_90,rcc_mean,
rcc_std,rcc_50,rcc_75,rcc_90,max_solar_elev,snow_flag,outlierflag_gcc_mean,
outlierflag_gcc_50,outlierflag_gcc_75,outlierflag_gcc_90
2008-04-04,04:01:40,95,harvard_2008_04_04_040140.jpg,-15.59448,-9999.0,1,0.18285
,24.76647,2.61462,20,22,23,25,26,28,29,11.22271,2.51949,7,8,10,11,13,14,15,25.38
766,2.74048,21,22,24,26,27,29,30,0.87536,0.84553,0.83233,50.,-9999,-9999,-9999,-9999,
-9999
2008-04-04,04:31:40,95,harvard_2008_04_04_043140.jpg,-10.42516,-9999.0,1,0.18382
,24.78447,2.62504,20,22,23,25,26,28,29,11.25862,2.54794,7,8,10,11,13,14,15,25.20
662,2.80735,20,22,24,25,27,28,29,0.88964,0.83442,0.82846,50.,-9999,-9999,-9999,-9999,
-9999

At some point we need to account for missing data.  One way would be
to make an empty row for that date.

2008-04-06,,,,,,,,,,,,,,,,

The following image selection parameters are read from a config file
(if it's present) or set to a default if not.  Only values changed
from defaults need to be in the config file.

default_nimage_threshold=1
default_time_min='00:00:00'
default_time_max='23:59:59'

Here's a sample config file which should be named
<sitename>_<roi-id>_gcc90.cfg and be in the ROI directory
of the data archive for the site:

----start of file----
[gcc90_calculation]
brt_threshold_min=160

----end of file----

You can specify any of the above parameters to configure
image selection in the gcc90 calculations.


"""

import argparse
import os
from ConfigParser import SafeConfigParser
from datetime import datetime
from datetime import time
from datetime import timedelta

import numpy as np

import vegindex as vi

from .quantile import quantile

# set vars

# you can set the archive directory to somewhere else for testing by
# using the env variable, PHENOCAM_ARCHIVE_DIR.
archive_dir = vi.config.archive_dir

# set missing/no-data values
ND_FLOAT = vi.config.ND_FLOAT
ND_INT = vi.config.ND_INT
ND_STRING = vi.config.ND_STRING

irflg = False
debug = False

# get default image selectors
default_nimage_threshold = vi.config.NIMAGE_MIN
default_time_min = vi.config.TIME_MIN
default_time_max = vi.config.TIME_MAX
default_sunelev_min = vi.config.MIN_SUN_ANGLE
default_brt_min = vi.config.MIN_BRT
default_brt_max = vi.config.MAX_BRT


def main():

    # set up command line argument processing
    parser = argparse.ArgumentParser()

    # options
    parser.add_argument("-v", "--verbose",
                        help="increase output verbosity",
                        action="store_true",
                        default=False)
    parser.add_argument("-n", "--dry-run",
                        help="Process data but don't save results",
                        action="store_true",
                        default=False)

    parser.add_argument("-p", "--aggregation-period",
                        help="Number of Days to Aggregate (default=1)",
                        nargs='?', type=int, choices=range(1, 5, 2),
                        default=1)

    # positional arguments
    parser.add_argument("site", help="PhenoCam site name")
    parser.add_argument("roiname", help="ROI name, e.g. canopy_0001")

    # get args
    args = parser.parse_args()
    sitename = args.site
    roiname = args.roiname
    verbose = args.verbose
    dryrun = args.dry_run
    ndays = args.aggregation_period

    if verbose:
        print "site: {0}".format(sitename)
        print "roiname: {0}".format(roiname)
        print "verbose: {0}".format(verbose)
        print "dryrun: {0}".format(dryrun)
        print "period: {0}".format(ndays)

    # read in config file for this ROI List if it exists
    config_file = "{0}_{1}.cfg".format(sitename, roiname)
    config_path = os.path.join(archive_dir, sitename, 'ROI',
                               config_file)
    if os.path.exists(config_path):
        # NOTE: should probably subclass safe config parser
        # and add gettime() method which checks for time validity
        cfgparser = SafeConfigParser(
            defaults={'nimage_threshold': str(default_nimage_threshold),
                      'time_min': str(default_time_min),
                      'time_max': str(default_time_max),
                      'sunelev_min': str(default_sunelev_min),
                      'brt_min': str(default_brt_min),
                      'brt_max': str(default_brt_max)})

        cfgparser.read(config_path)

        if cfgparser.has_section('gcc90_calculation'):
            nimage_threshold = cfgparser.getint('gcc90_calculation',
                                                'nimage_threshold')
            time_max_str = cfgparser.get('gcc90_calculation',
                                         'time_max')
            [tmax_hr, tmax_mn, tmax_sc] = time_max_str.split(':')
            time_max = time(int(tmax_hr), int(tmax_mn), int(tmax_sc))
            time_min_str = cfgparser.get('gcc90_calculation',
                                         'time_min')
            [tmin_hr, tmin_mn, tmin_sc] = time_min_str.split(':')
            time_min = time(int(tmin_hr), int(tmin_mn), int(tmin_sc))
            sunelev_min = cfgparser.getfloat('gcc90_calculation',
                                             'sunelev_min')
            brt_min = cfgparser.getint('gcc90_calculation', 'brt_min')
            brt_max = cfgparser.getint('gcc90_calculation', 'brt_max')
        else:
            nimage_threshold = int(default_nimage_threshold)
            [tmax_hr, tmax_mn, tmax_sc] = default_time_max.split(':')
            time_max = time(int(tmax_hr), int(tmax_mn), int(tmax_sc))
            [tmin_hr, tmin_mn, tmin_sc] = default_time_min.split(':')
            time_min = time(int(tmin_hr), int(tmin_mn), int(tmin_sc))
            sunelev_min = default_sunelev_min
            brt_min = default_brt_min
            brt_max = default_brt_max

    else:
        nimage_threshold = int(default_nimage_threshold)
        [tmax_hr, tmax_mn, tmax_sc] = default_time_max.split(':')
        time_max = time(int(tmax_hr), int(tmax_mn), int(tmax_sc))
        [tmin_hr, tmin_mn, tmin_sc] = default_time_min.split(':')
        time_min = time(int(tmin_hr), int(tmin_mn), int(tmin_sc))
        sunelev_min = default_sunelev_min
        brt_min = default_brt_min
        brt_max = default_brt_max

    # print config values
    if verbose:
        print ''
        print 'gcc config:'
        print '==========='
        print 'roi_list: ', '{0}_{1}_roi.csv'.format(sitename, roiname)
        if os.path.exists(config_path):
            print "config file: {0}".format(config_file)
        else:
            print "config file: None"
        print 'nimage threshold: ', nimage_threshold
        print 'time of day min: ', time_min
        print 'time of day max: ', time_max
        print 'sun elev min: ', sunelev_min
        print 'aggregate days: ', ndays
        print 'minimum brightness: ', brt_min
        print 'maximum brightness: ', brt_max

    # set up output filename
    outdir = os.path.join(archive_dir, sitename, 'ROI')
    outfile = '{0}_{1}_{2}day.csv'.format(sitename, roiname, ndays)
    outpath = os.path.join(outdir, outfile)
    if verbose:
        print 'output file: ', outfile

    # create gcc timeseries object as empty list
    gcc_ts = vi.GCCTimeSeries(site=sitename, ROIListID=roiname,
                              nday=ndays,
                              nmin=nimage_threshold,
                              tod_min=time_min,
                              tod_max=time_max,
                              sunelev_min=sunelev_min,
                              brt_min=brt_min,
                              brt_max=brt_max)

    # get roi timeseries for this site and roi
    roits = vi.get_roi_timeseries(sitename, roiname)

    if verbose:
        print ''
        print 'ROI timeseries info:'
        print '===================='
        print 'site: ', roits.site
        print 'ROI list id: ', roits.roilistid
        print 'create date: ', roits.created_at
        print 'update date: ', roits.updated_at
        print 'nrows: ', len(roits.rows)

    # make list of rows which match image selection criteria
    roits_rows = roits.select_rows(tod_min=time_min,
                                   tod_max=time_max,
                                   sunelev_min=sunelev_min,
                                   brt_min=brt_min,
                                   brt_max=brt_max)

    # check that some rows passed selection criteria
    nrows = len(roits_rows)
    if nrows == 0:
        print "No rows passed the selection criteria"
        return

    if verbose:
        print 'Number of selected rows: {0}'.format(nrows)

    # make a list of dates for selected images
    img_date = []
    for row in roits_rows:
        img_date.append(row['datetime'].date())

    # list is ordered so find first and last dates
    dt_first = img_date[0]
    dt_last = img_date[nrows - 1]

    # set up a generator which yields dates for the start
    # of the next nday period covering the date range of image
    gcc_dr = vi.daterange2(dt_first, dt_last, ndays)

    # calculate offset for timeseries based on nday
    day_offset = ndays / 2
    date_offset = timedelta(days=day_offset)

    # roits_ndx will be index into ROI timeseries
    roits_ndx = 0

    # set up vars for accumulating stats
    img_cnt = 0
    filenames = []
    r_dn_vals = []
    rcc_vals = []
    g_dn_vals = []
    gcc_vals = []
    b_dn_vals = []
    bcc_vals = []
    solar_elev_vals = []
    midday_delta_vals = []

    # loop over nday time periods
    for gcc_ndx, start_date in enumerate(gcc_dr):

        end_date = start_date + timedelta(ndays)
        gcc_date = start_date + date_offset
        doy = gcc_date.timetuple().tm_yday
        midday_noon = datetime(gcc_date.year,
                               gcc_date.month,
                               gcc_date.day, 12, 0, 0)

        # get roits rows for this time period
        while (roits_ndx < nrows and
               img_date[roits_ndx] >= start_date and
               img_date[roits_ndx] < end_date):
            filenames.append(roits_rows[roits_ndx]['filename'])
            r_dn = roits_rows[roits_ndx]['r_mean']
            r_dn_vals.append(r_dn)
            g_dn = roits_rows[roits_ndx]['g_mean']
            g_dn_vals.append(g_dn)
            b_dn = roits_rows[roits_ndx]['b_mean']
            b_dn_vals.append(b_dn)
            dnsum = r_dn + g_dn + b_dn

            # NOTE: I'm recomputing gcc, rcc, bcc from DN values rather
            # than using value stored in all-image CSV
            if dnsum <= 0:
                rcc = np.nan
                bcc = np.nan
                gcc = np.nan
            else:
                img_cnt += 1
                rcc = r_dn / dnsum
                bcc = b_dn / dnsum
                gcc = roits_rows[roits_ndx]['gcc']

            solar_elev = roits_rows[roits_ndx]['solar_elev']

            # note that rcc_vals can include NaN's
            rcc_vals.append(rcc)
            gcc_vals.append(gcc)
            bcc_vals.append(bcc)
            solar_elev_vals.append(solar_elev)
            midday_td = roits_rows[roits_ndx]['datetime'] - midday_noon
            midday_td_secs = np.abs(midday_td.days * 86400 + midday_td.seconds)
            midday_delta_vals.append(midday_td_secs)

            if roits_ndx < nrows:
                roits_ndx += 1
            else:
                break

        # check to see if we got any (good) images
        if img_cnt == 0:
            # nodata for this time period
            image_count = 0
            midday_filename = ND_STRING
            midday_r = ND_FLOAT
            midday_g = ND_FLOAT
            midday_b = ND_FLOAT
            midday_gcc = ND_FLOAT
            midday_rcc = ND_FLOAT
            r_mean = ND_FLOAT
            r_std = ND_FLOAT
            g_mean = ND_FLOAT
            g_std = ND_FLOAT
            b_mean = ND_FLOAT
            b_std = ND_FLOAT
            gcc_mean = ND_FLOAT
            gcc_std = ND_FLOAT
            gcc_50 = ND_FLOAT
            gcc_75 = ND_FLOAT
            gcc_90 = ND_FLOAT
            rcc_mean = ND_FLOAT
            rcc_std = ND_FLOAT
            rcc_50 = ND_FLOAT
            rcc_75 = ND_FLOAT
            rcc_90 = ND_FLOAT
            max_solar_elev = ND_FLOAT
            snow_flag = ND_INT
            outlierflag_gcc_mean = ND_INT
            outlierflag_gcc_50 = ND_INT
            outlierflag_gcc_75 = ND_INT
            outlierflag_gcc_90 = ND_INT

        # got some good images but not enough - probably there
        # are cases where this will fail e.g. not images on the
        # midday of a 3-day aggregation period.
        elif img_cnt < nimage_threshold:
            # not enough images
            image_count = img_cnt
            # find nearest image to midday (noon) on mid-interval date
            mi_ndx = midday_delta_vals.index(min(midday_delta_vals))
            midday_filename = filenames[mi_ndx]
            midday_r = r_dn_vals[mi_ndx]
            midday_g = g_dn_vals[mi_ndx]
            midday_b = b_dn_vals[mi_ndx]
            midday_gcc = gcc_vals[mi_ndx]
            midday_rcc = rcc_vals[mi_ndx]

            # no stats for this time interval
            r_mean = ND_FLOAT
            r_std = ND_FLOAT
            g_mean = ND_FLOAT
            g_std = ND_FLOAT
            b_mean = ND_FLOAT
            b_std = ND_FLOAT
            gcc_mean = ND_FLOAT
            gcc_std = ND_FLOAT
            gcc_50 = ND_FLOAT
            gcc_75 = ND_FLOAT
            gcc_90 = ND_FLOAT
            rcc_mean = ND_FLOAT
            rcc_std = ND_FLOAT
            rcc_50 = ND_FLOAT
            rcc_75 = ND_FLOAT
            rcc_90 = ND_FLOAT
            max_solar_elev = max(solar_elev_vals)
            snow_flag = ND_INT
            outlierflag_gcc_mean = ND_INT
            outlierflag_gcc_50 = ND_INT
            outlierflag_gcc_75 = ND_INT
            outlierflag_gcc_90 = ND_INT

        # stats for this period should be complete - only
        # snow flags are missing data
        else:
            # find nearest image to midday (noon) on mid-interval date
            mi_ndx = midday_delta_vals.index(min(midday_delta_vals))
            midday_filename = filenames[mi_ndx]
            midday_r = r_dn_vals[mi_ndx]
            midday_g = g_dn_vals[mi_ndx]
            midday_b = b_dn_vals[mi_ndx]
            midday_gcc = gcc_vals[mi_ndx]
            midday_rcc = rcc_vals[mi_ndx]

            # get stats for this time interval
            image_count = img_cnt
            r_mean = np.nanmean(r_dn_vals)
            r_std = np.nanstd(r_dn_vals)
            g_mean = np.nanmean(g_dn_vals)
            g_std = np.nanstd(g_dn_vals)
            b_mean = np.nanmean(b_dn_vals)
            b_std = np.nanstd(b_dn_vals)
            gcc_mean = np.nanmean(gcc_vals)
            gcc_std = np.nanstd(gcc_vals)
            gcc_50 = quantile(gcc_vals, .5)
            gcc_75 = quantile(gcc_vals, .75)
            gcc_90 = quantile(gcc_vals, .9)
            rcc_mean = np.nanmean(rcc_vals)
            rcc_std = np.nanstd(rcc_vals)
            rcc_50 = quantile(rcc_vals, .5)
            rcc_75 = quantile(rcc_vals, .75)
            rcc_90 = quantile(rcc_vals, .9)
            max_solar_elev = max(solar_elev_vals)
            snow_flag = ND_INT
            outlierflag_gcc_mean = ND_INT
            outlierflag_gcc_50 = ND_INT
            outlierflag_gcc_75 = ND_INT
            outlierflag_gcc_90 = ND_INT

        # append to gcc timeseries
        gcc_ts_row = gcc_ts.insert_row(gcc_date, doy, image_count,
                                       midday_filename,
                                       midday_r,
                                       midday_g,
                                       midday_b,
                                       midday_gcc,
                                       midday_rcc,
                                       r_mean, r_std,
                                       g_mean, g_std,
                                       b_mean, b_std,
                                       gcc_mean, gcc_std,
                                       gcc_50, gcc_75, gcc_90,
                                       rcc_mean, rcc_std,
                                       rcc_50, rcc_75, rcc_90,
                                       max_solar_elev, snow_flag,
                                       outlierflag_gcc_mean,
                                       outlierflag_gcc_50,
                                       outlierflag_gcc_75,
                                       outlierflag_gcc_90)

        # print result if verbose
        if verbose:
            csvstr = gcc_ts.format_csvrow(gcc_ts_row)
            print csvstr

        # reset accumulated values
        img_cnt = 0
        filenames = []
        r_dn_vals = []
        rcc_vals = []
        g_dn_vals = []
        gcc_vals = []
        b_dn_vals = []
        bcc_vals = []
        solar_elev_vals = []
        midday_delta_vals = []

    if dryrun:
        nout = 0
    else:
        nout = gcc_ts.writeCSV(outpath)

    print 'Total: %d' % (nout,)


# run main when called from command line
if __name__ == "__main__":
    main()
